NAME = "story.py"
ORIGINAL_AUTHORS = ["Tuan Thai"]

ABOUT = """
Sends a random story
Sourced stories from
http://www.read.gov/aesop/001.html
"""

COMMANDS = """
>>> .story
returns random story
"""

WEBSITE = ""



"""
[debug.py]
Debug plugin : Un

[Author]
Abdur-Rahmaan Janhangeer, pythonmembers.club

[About]
prints all parameters passed to bot

[Commands]
>>> .debug
prints some parameters
"""


class Plugin:
    def __init__(self):
        pass

    def run(self, incoming, methods, info, bot_info):
        try:
            # if '!~' in info['prefix']:
            # print(info)
            if info["command"] == "PRIVMSG" and info["args"][1] == ".debug":
                methods["send"](info["address"], info["prefix"])
        except Exception as e:
            print("woops plug", e)



"""
[greet.py]
Greet Plugin

[Author]
Abdur-Rahmaan Janhangeer, pythonmembers.club

[About]
responds to .hi, demo of a basic plugin

[Commands]
>>> .hi
returns hoo
"""


class Plugin:
    def __init__(self):
        pass

    def run(self, incoming, methods, info, bot_info):

        try:
            # if '!~' in info['prefix']:
            # print(info)
            if info["command"] == "PRIVMSG" and info["args"][1] == ".hi":
                methods["send"](info["address"], "hooo")
        except Exception as e:
            print("woops plug", e)



NAME = "movies_imdb.py"
ORIGINAL_AUTHORS = ["Zoi Katsantoni"]

ABOUT = """
returns a certain information about a movie from IMDb
eg. cast, directors,...
"""

COMMANDS = """
>>> .movies_imdb <<search>> <<movie>> <<information>>
returns the information for the movie
.movies_imdb 250movies
returns 250 all-time most popular movies
"""

WEBSITE = ""



def set_nick(name):
    return f"NICK {name} \r\n"


def present(name):
    return "USER {0} {0} {0} : {0} IRC\r\n".format(name)


def identify(password):
    return f"msg NickServ identify {password} \r\n"


def join_channel(channel):
    return f"JOIN {channel} \r\n"


def specific_send(target, msg):
    return f"PRIVMSG {target} :{msg}\r\n"


def pong_return(domain):
    return f"PONG :{domain}\r\n"


def quit():
    return f"QUIT Leaving / \r\n"



"""
[monopoly.py]
Monopoly Plugin
[Author]
Angelo Giacco
[About]
Play monopoly
[Commands]
>>>.monopoly create
creates monopoly

>>>.monopoly join
user joins the game

>>>.monopoly start
starts the game

>>>.monopoly roll
rolls two dice to see how far the user will move/if the user can leave the jail

>>>.monopoly buy
user purchases property if unowned, otherwise buys house

>>>.monopoly pass
user passes on the opportunity to buy a property if unowned,
or on the opportunity to buy a house

>>>.monopoly leave
user leaves the game

>>>.monopoly help
sends messages explaining how to use the monopoly plugin

>>>.monopoly info
sends messages showing the status of each player

.monopoly info rolls
displays basic information about the upcoming spaces in front of a player

.monopoly info <<player>>
shows information about a certain player

.monopoly info <<property>>
shows information about a property
"""

import random

import monopoly_assets.monopoly_assets
import monopoly_assets.monopoly_player


class Plugin:
    """class variables"""

    stage = None  # none for no game yet initalised, 0 for create, 1 for already started
    game_over = False  # becomes true when only one player left
    players = []  # stores player objects
    create_req = True  # create command required
    start_join_req = False  # start game or join game required
    roll_req = False  # roll required
    buy_pass_req = False  # buy or pass required
    winner = ""
    turn = 0

    def __init__(self):
        pass

    """
    USEFUL FUNCTIONS
    """

    def next_turn(self):
        """increment turn and set to start if will cause index error"""
        try:
            Plugin.turn += 1
            if Plugin.turn == len(Plugin.players):
                Plugin.turn -= len(Plugin.players)
        except Exception as e:
            print("woops, monopoly turn change error ", e)

    def checkWon(methods, info):
        """check if a user has won after game has started"""
        if len(Plugin.players) == 1 and Plugin.stage > 0:
            Plugin.winner = Plugin.players[0].getName()
            Plugin.game_over = True
            methods["send"](info["address"], "The winner of this monopoly game is " + Plugin.winner)
            return True
        else:
            return False

    def count_color_property(portfolio, col):
        """counts number of properties of a certain colour from a given portfolio"""
        properties = [
            asset for asset in portfolio if isinstance(asset, monopoly_assets.Property)
        ]  # util and railroad do not have color
        return sum(property.color == col for property in properties)  # True is equal to 1 in python

    def count_utilities(portfolio):
        """counts number of utilities in a given portfolio"""
        return len(
            [utility for utility in portfolio if isinstance(utility, monopoly_assets.Utility)]
        )

    def count_railroads(portfolio):
        """counts number of railroads in a given portfolio"""
        return len(
            [railroad for railroad in portfolio if isinstance(railroad, monopoly_assets.Railroad)]
        )

    def count_player_properties(player):
        """count property types of assets in player portfolio, returns a dict"""
        portfolio = player.getPortfolio()

        violet = Plugin.count_color_property(portfolio, "Violet")
        light_blue = Plugin.count_color_property(portfolio, "Light blue")
        purple = Plugin.count_color_property(portfolio, "Purple")
        orange = Plugin.count_color_property(portfolio, "Orange")
        red = Plugin.count_color_property(portfolio, "Red")
        yellow = Plugin.count_color_property(portfolio, "Yellow")
        green = Plugin.count_color_property(portfolio, "Green")
        blue = Plugin.count_color_property(portfolio, "Blue")
        utilities = Plugin.count_utilities(portfolio)
        railroads = Plugin.count_railroads(portfolio)
        prop_count_dict = {
            "Violet": violet,
            "Light blue": light_blue,
            "Purple": purple,
            "Orange": orange,
            "Red": red,
            "Yellow": yellow,
            "Green": green,
            "Blue": blue,
            "Utility": utilities,
            "Railroad": railroads,
        }
        return prop_count_dict

    def check_player(player_name):
        for player in Plugin.players:
            if player.getName() == player_name:
                return player
        return False

    def show_rolls(methods, info, player):
        """show what will happen for each amount a user rolls"""
        methods["send"](
            info["address"],
            "Upcoming spaces that " + player.getName() + " could roll to:",
        )
        start = player.getPosition()
        for i in range(2, 13):
            index = start + i
            if index > 39:
                index -= 40
            space = monopoly_assets.board_spaces[index]
            if (
                isinstance(space, monopoly_assets.Property)
                or isinstance(space, monopoly_assets.Railroad)
                or isinstance(space, monopoly_assets.Utility)
            ):
                owner_info = Plugin.find_owner(space.get_name())
                if owner_info[0]:
                    if owner_info[1] == player:
                        message = (
                            "If you roll "
                            + str(i)
                            + " you will end up at "
                            + space.get_name()
                            + " which you own"
                        )
                        if isinstance(space, monopoly_assets.Property):
                            message = (
                                message
                                + " and can buy a house for if you"
                                + " have a full set and don't already have a hotel."
                            )
                    else:
                        rent = Plugin.get_rent(space, owner_info[1], i)
                        message = (
                            "If you roll "
                            + str(i)
                            + " you will end up at "
                            + space.get_name()
                            + " which you is owned by "
                            + owner_info[1].getName()
                            + " and you would have to pay them "
                            + str(rent)
                        )
                else:
                    message = (
                        "If you roll "
                        + str(i)
                        + " you will end up at "
                        + space.get_name()
                        + " which is unowned and can be bought for"
                        + " "
                        + str(space.price)
                    )
            else:
                message = (
                    "If you roll "
                    + str(i)
                    + " you will end up at "
                    + space.get_name()
                    + " which you own"
                )
            methods["send"](info["address"], message)

    def get_player_info(methods, info, player):
        portfolio = player.getPortfolio()  # returns array
        name = player.getName()
        pot = player.getPot()  # returns string
        methods["send"](info["address"], name + "'s pot is " + pot)
        if len(portfolio) > 0:
            portfolio_dict = Plugin.count_player_properties(player)
            print(portfolio_dict)
            print(portfolio_dict.keys())
            for key in portfolio_dict.keys():

                if portfolio_dict[key] != 0:
                    message = name + " has " + str(portfolio_dict[key]) + " " + key + " properties"
                    methods["send"](info["address"], message)
                else:
                    continue

        else:
            methods["send"](info["address"], name + " has no properties")

        if player.getPosition() == 10 and player.imprisoned:
            methods["send"](info["address"], name + " is locked in jail")
        elif player.getPosition() == 10 and not player.imprisoned:
            methods["send"](info["address"], name + " is at the jail but not inside... yet")
        else:
            location = monopoly_assets.board_spaces[player.getPosition()].get_name()
            methods["send"](
                info["address"],
                name
                + " is at "
                + location
                + " which is board space number "
                + str(player.getPosition()),
            )

    def get_info(methods, info):
        """get_info about players"""
        if Plugin.winner != "" and Plugin.game_over:
            methods["send"](info["address"], "The game is over and the winner was " + Plugin.winner)
        elif len(Plugin.players) > 0:
            for player in Plugin.players:
                Plugin.get_player_info(methods, info, player)
            methods["send"](
                info["address"],
                "It is " + Plugin.players[Plugin.turn].getName() + "'s turn!",
            )
        else:
            methods["send"](info["address"], "There are no players yet!")

    def get_help(methods, info):
        """print help messages"""
        methods["send"](info["address"], "'.monopoly create' creates a new game people can join.")
        methods["send"](
            info["address"],
            "'.monopoly start' starts the game. People can no longer join.",
        )
        methods["send"](info["address"], "'.monopoly roll' rolls two dice and move on the board.")
        methods["send"](
            info["address"],
            "'.monopoly buy' buys the property where you are on the board "
            "or buy a house for the property.",
        )
        methods["send"](
            info["address"],
            "'.monopoly pass' refuses to buy the property or house for the property.",
        )
        methods["send"](info["address"], "'.monopoly leave' leaves the game")
        methods["send"](info["address"], "'.monopoly gameinfo' gets information about each player.")
        methods["send"](info["address"], "Hope this helps!")

    def find_owner(asset_name):
        """
        find the player object that has an asset in its portfolio, must pass ASSET NAME
        """
        for player in Plugin.players:
            for property in player.getPortfolio():
                if property.get_name() == asset_name:
                    return [True, player]
        return [False]

    def get_rent(asset, asset_owner, move_amount):
        """find the rent of an asset"""
        if isinstance(asset, monopoly_assets.Property):
            return asset.rents[asset.house_count]

        elif isinstance(asset, monopoly_assets.Utility):
            num_util = len(
                [
                    property
                    for property in asset_owner.getPortfolio()
                    if isinstance(property, monopoly_assets.Utility)
                ]
            )
            print(num_util)
            calc_string = asset.rents[num_util] + " " + str(move_amount)
            print(calc_string)
            return eval(  # nosec eval_used
                calc_string,
                {"__builtins__": None},
                {"str": str, "int": int, "float": float},
            )

        elif isinstance(asset, monopoly_assets.Railroad):
            num_rail = len(
                [
                    property
                    for property in asset_owner.getPortfolio()
                    if isinstance(property, monopoly_assets.Railroad)
                ]
            )
            print(num_rail)
            return asset.rents[num_rail]
        else:
            return 0

    def move_to_space(methods, info, player, space):
        """move a player to a space object"""
        if monopoly_assets.board_spaces.index(space) < player.getPosition() <= 39:
            methods["send"](
                info["address"],
                player.getName() + " passed go moving to " + space.get_name(),
            )
            player.increasePot(200)
            player.setPosition(monopoly_assets.board_spaces.index(space))
        elif player.getPosition == monopoly_assets.board_spaces.index(space):
            methods["send"](
                info["address"],
                player.getName() + " was already at " + space.get_name(),
            )
        else:
            methods["send"](info["address"], player.getName() + " moved to " + space.get_name())
            player.setPosition(monopoly_assets.board_spaces.index(space))

    def get_location_info(methods, info, player, location):  # location is a space object
        """
        get information about a property and whether other properties in the set are owned
        """
        if isinstance(location, monopoly_assets.Property):
            methods["send"](info["address"], location.info())
            methods["send"](
                info["address"],
                "Remember that you need a full set to buy houses! "
                "Of the other properties in the set:",
            )
            set_others = []
            for space in monopoly_assets.board_spaces:
                if isinstance(space, monopoly_assets.Property):
                    if space.color == location.color and space != location:
                        set_others.append(space)
            for space in set_others:
                owner_info = Plugin.find_owner(space.get_name())
                if owner_info[0]:
                    if owner_info[1].getName() == player.getName():
                        methods["send"](info["address"], "You already own " + space.get_name())
                    else:
                        methods["send"](
                            info["address"],
                            space.get_name() + " is owned by " + owner_info[1].getName(),
                        )
                else:
                    methods["send"](info["address"], space.get_name() + " is unowned!")

        elif isinstance(location, monopoly_assets.Railroad):
            methods["send"](info["address"], location.info())
            other_rails = []
            for space in monopoly_assets.board_spaces:
                if isinstance(space, monopoly_assets.Railroad) and space != location:
                    other_rails.append(space)
            methods["send"](info["address"], "Of the other railroads:")
            for rail in other_rails:
                owner_info = Plugin.find_owner(rail.get_name())
                if owner_info[0]:
                    if owner_info[1].getName() == player.getName():
                        methods["send"](info["address"], "You already own " + rail.get_name())
                    else:
                        methods["send"](
                            info["address"],
                            rail.get_name() + " is owned by " + owner_info[1].getName(),
                        )
                else:
                    methods["send"](info["address"], rail.get_name() + " is unowned!")

        elif isinstance(location, monopoly_assets.Utility):
            methods["send"](info["address"], location.info())
            for space in monopoly_assets.board_spaces:
                if isinstance(space, monopoly_assets.Utility) and space != location:
                    other_util = space
                    break
            owner_info = Plugin.find_owner(other_util.get_name())
            if owner_info[0]:
                if owner_info[1].getName() == player.getName():
                    methods["send"](info["address"], "You already own " + other_util.get_name())
                else:
                    methods["send"](
                        info["address"],
                        other_util.get_name() + " is owned by " + owner_info[1].getName(),
                    )
            else:
                methods["send"](info["address"], other_util.get_name() + " is unowned!")
        else:
            methods["send"](info["address"], "No special information can be given for this space!")

    def land_unowned_property(methods, info, player, space):
        """method for landing on an unowned property"""
        methods["send"](
            info["address"],
            space.get_name()
            + " is unowned and may be"
            + " bought for "
            + str(space.price)
            + ". Enter '.monopoly buy' if you "
            + "want to to buy it or '.monopoly pass' if not",
        )
        Plugin.roll_req = False
        Plugin.buy_pass_req = True
        Plugin.get_location_info(methods, info, player, space)

    def pay(methods, info, payer, receiver, amount):
        """payers pays a user a certain amount and
        may be kicked out of the game if has no money"""
        payer_alive = payer.reducePot(amount)
        receiver.increasePot(amount)

        if payer_alive:
            methods["send"](
                info["address"],
                payer.getName()
                + " now only has "
                + payer.getPot()
                + " whereas "
                + receiver.getName()
                + " now has "
                + receiver.getPot(),
            )
            Plugin.next_turn()

        else:
            methods["send"](
                info["address"],
                payer.getName()
                + " is now out of the game "
                + " whereas "
                + receiver.getName()
                + " now has "
                + receiver.getPot(),
            )
            Plugin.leave(payer.getName())
            Plugin.checkWon(methods, info)
            Plugin.next_turn()

    def offer_house(methods, info, player, property):
        """offer the player a house"""
        num_prop_in_set = Plugin.count_player_properties(player)[property.color]
        if num_prop_in_set == monopoly_assets.Property.set_houses[property.color]:
            if property.house_count == 5:
                methods["send"](info["address"], "Your property already has a hotel!")
                Plugin.next_turn()
            else:
                cost_house = property.get_house_cost()
                methods["send"](
                    info["address"],
                    "You have landed on your own property "
                    + "and can buy a house for "
                    + str(cost_house),
                )
                Plugin.buy_pass_req = True
                Plugin.roll_req = False
        else:
            methods["send"](
                info["address"],
                "You have landed on your own property."
                + "but cannot buy a house as you need a full set",
            )
            Plugin.next_turn()

    """
    COMMUNITY CARD AND CHANCE METHODS
    """

    # all of these methods will take self,methods,info,player arguments for simplicity
    # additional arguments supplied can be found in the second element of the values
    # of the dictionaries of the monopoly_assets.py file.
    # here's an example
    # {"From sale of stock, you get $45":[0,(45)]}
    # 0 refers to earn as it is the function at first index of community_deck_methods
    # and (45) is the argument.
    # additional arugments will be supplied using * so that they are split up

    def community_card(methods, info, player):
        """pick a community card from top, execute it, add it to bottom of pack"""
        # take community Card
        # must pick first in array, place it at back,
        # and execute its function with correct args.
        community_card_methods = [
            Plugin.earn,
            Plugin.get_outta_jail,
            Plugin.collect,
            Plugin.go,
            Plugin.jail,
            Plugin.repairs,
            Plugin.fine,
        ]
        methods["send"](
            info["address"],
            player.getName() + " is picking up a community card, it says:",
        )
        card = monopoly_assets.community_deck.pop(
            0
        )  # take first card, returns a dict {"message":[int,(tuple)]}
        card_string = next(iter(card))
        methods["send"](info["address"], card_string)
        # execute card: get function, add standard arguments,
        # unpack those stored in the thing
        func = community_card_methods[card[card_string][0]]
        func(methods, info, player, *card[card_string][1])
        monopoly_assets.community_deck.append(card)  # put it at the back

    def chance_card(methods, info, player):
        """pick a chance card from top, execute it, add it to bottom of pack"""
        chance_card_methods = [
            Plugin.pay_all,
            Plugin.reading_rail,
            Plugin.move_to_property,
            Plugin.go,
            Plugin.railroad,
            Plugin.get_outta_jail,
            Plugin.jail,
            Plugin.earn,
            Plugin.fine,
            Plugin.repairs,
            Plugin.util,
            Plugin.move_back_three,
        ]
        methods["send"](
            info["address"], player.getName() + " is picking up a chance card, it says:"
        )
        card = monopoly_assets.chance_deck.pop(
            0
        )  # take first card, returns a dict {"message":[int,(tuple)]}
        card_string = next(iter(card))
        methods["send"](info["address"], card_string)
        # execute card: get function, add standard arguments,
        # unpack those stored in the thing
        func = chance_card_methods[card[card_string][0]]
        func(methods, info, player, *card[card_string][1])
        monopoly_assets.chance_deck.append(card)  # put it at the back

    def earn(methods, info, player, amount):
        """generate money, not taken from someone"""
        player.increasePot(amount)
        methods["send"](info["address"], player.getName() + " now has a pot of " + player.getPot())
        Plugin.next_turn()

    def get_outta_jail(methods, info, player):
        """get out of jail if imprisoned, store get out of jail card if not"""
        if player.imprisoned:
            player.imprisoned = False
            player.prison_time = 0
            methods["send"](info["address"], player.getName() + " escaped the jail")
            Plugin.next_turn()
        else:
            player.get_outta_jail_card = True
            Plugin.next_turn()

    def collect(methods, info, player, amount):
        """collect amount from all players,
        can't use def pay as will execute next_turn too many times"""
        for opponent in Plugin.players:  # will iterate over player as well
            if player != opponent:
                payer_alive = opponent.reducePot(amount)
                player.increasePot(amount)

                if payer_alive:
                    methods["send"](
                        info["address"],
                        opponent.getName()
                        + " now only has "
                        + opponent.getPot()
                        + " whereas "
                        + player.getName()
                        + " now has "
                        + player.getPot(),
                    )

                else:
                    methods["send"](
                        info["address"],
                        opponent.getName()
                        + " is now out of the game "
                        + " whereas "
                        + player.getName()
                        + " now has "
                        + player.getPot(),
                    )
                    Plugin.leave(opponent.getName())
                    Plugin.checkWon(methods, info)
        Plugin.next_turn()

    def go(methods, info, player):
        """move to go and earn 200"""
        player.setPosition(0)
        player.increasePot(200)
        methods["send"](
            info["address"],
            player.getName()
            + " moved to GO, earning 200"
            + " so their pot is now "
            + player.getPot(),
        )
        Plugin.next_turn()

    def jail(methods, info, player):
        """
        remove get out of jail card if player has it,
        otherwise nd to jail, do not gain 200
        """
        if player.get_outta_jail_card:
            methods["send"](
                info["address"],
                player.getName() + " is at jail but used his get out of jail free card!",
            )
            player.setPosition(10)
            Plugin.next_turn()
        else:
            methods["send"](info["address"], player.getName() + " has been sent to jail!")
            player.setPosition(10)
            player.imprisoned = True
            player.prison_time = 0
            Plugin.next_turn()

    def fine(methods, info, player, amount):
        """
        reduce a certain amount of money from a player and check if player has survived
        """
        player_alive = player.reducePot(amount)

        if player_alive:
            methods["send"](
                info["address"],
                player.getName()
                + " was forced to pay "
                + str(amount)
                + " and now has "
                + player.getPot(),
            )
        else:
            methods["send"](
                info["address"],
                player.getName()
                + " was forced to pay "
                + str(amount)
                + " on and now has no more money so has left the game",
            )
            Plugin.leave(player.getName())
            Plugin.checkWon(methods, info)
        Plugin.next_turn()

    def repairs(methods, info, player, house_fee, hotel_fee):
        """calculate repairs fee and then fine player that amount"""
        property_list = [
            asset for asset in player.getPortfolio() if isinstance(asset, monopoly_assets.Property)
        ]
        hotel_total = 0
        house_total = 0
        for property in property_list:
            if property.house_count == 5:
                hotel_total += 1
            else:
                house_total += property.house_count
        cost = house_fee * house_total + hotel_fee * hotel_total
        Plugin.fine(methods, info, player, cost)

    def pay_all(methods, info, player, amount):
        """check if player can pay the amount, remove if not,
        and increment pot of each opponent"""
        other_players = [opponent for opponent in Plugin.players if opponent != player]
        total_amount = amount * len(other_players)
        player_alive = player.reducePot(total_amount)

        if player_alive:
            methods["send"](
                info["address"],
                player.getName()
                + " was forced to pay "
                + str(amount)
                + " and now has "
                + player.getPot(),
            )
        else:
            methods["send"](
                info["address"],
                player.getName()
                + " was forced to pay "
                + str(amount)
                + " on repairs and now has no more money so has left the game",
            )
            Plugin.leave(player.getName())
            Plugin.checkWon(methods, info)

        for opponent in other_players:
            opponent.increasePot(amount)
        Plugin.next_turn()

    def reading_rail(methods, info, player):
        """move player to reading railroad and check if owned by player,
        owned by opponent or unowned"""
        reading_rail = monopoly_assets.board_spaces[5]
        Plugin.move_to_space(methods, info, player, reading_rail)
        owner_info = Plugin.find_owner("Reading Railroad")
        if owner_info[0]:
            # owned
            if owner_info[1] == player:
                methods["send"](
                    info["address"],
                    "You own Reading Railroad, but "
                    + "can not buy a house as railroads do not have houses!",
                )
                Plugin.next_turn()
            else:
                owner_railroads = Plugin.count_player_properties(owner_info[1])["Railroad"]
                rent = reading_rail.rents[owner_railroads]
                methods["send"](
                    info["address"],
                    "Reading Railroad is owned by"
                    + owner_info[1].getName()
                    + " and so "
                    + player.getName()
                    + " must pay them "
                    + str(rent),
                )
                Plugin.pay(methods, info, player, owner_info[1], rent)
        else:
            # unowned
            Plugin.land_unowned_property(methods, info, player, reading_rail)

    def railroad(methods, info, player):
        """move to nearest railroad and check if owned by player,
        owned by opponent or unowned"""
        # find railroad, all railroads have index ending in five.
        # in addition, player must move forwards.
        # if already at railroad must still go forward to next one
        for i in range(10):  # player can only be 10 away from next railroad
            player.updatePosition(1)
            if isinstance(
                monopoly_assets.board_spaces[player.getPosition()],
                monopoly_assets.Railroad,
            ):
                break
        railroad = monopoly_assets.board_spaces[player.getPosition()]
        owner_info = Plugin.find_owner(railroad.get_name())
        if owner_info[0]:
            if owner_info[1] == player:
                methods["send"](
                    info["address"],
                    player.getName() + " is at " + railroad.get_name() + " which they already own",
                )
                Plugin.next_turn()
            else:
                owner_railroads = Plugin.count_player_properties(owner_info[1])["Railroad"]
                rent = railroad.rents[owner_railroads]
                rent *= 2
                methods["send"](
                    info["address"],
                    railroad.get_name()
                    + " is owned by"
                    + owner_info[1].getName()
                    + " and so "
                    + player.getName()
                    + " must pay them "
                    + str(rent),
                )
                Plugin.pay(methods, info, player, owner_info[1], rent)
        else:
            Plugin.land_unowned_property(methods, info, player, railroad)

    def move_to_property(methods, info, player, property):
        """move to specific property and and check if owned by player,
        owned by opponent or unowned"""
        Plugin.move_to_space(methods, info, player, property)
        owner_info = Plugin.find_owner(property.get_name())
        if owner_info[0]:
            if owner_info[1] == player:
                Plugin.offer_house(methods, info, player, property)
            else:
                rent = property.rents[property.house_count]
                methods["send"](
                    info["address"],
                    property.get_name()
                    + " is owned by"
                    + owner_info[1].getName()
                    + " and so "
                    + player.getName()
                    + " must pay them "
                    + str(rent),
                )
                Plugin.pay(methods, info, player, owner_info[1], rent)
        else:
            Plugin.land_unowned_property(methods, info, player, property)

    def util(methods, info, player):
        """
        move to nearest utility and and check if owned by player,
        owned by opponent or unowned (should pay double rent)
        """
        start = False  # make sure the user advances at least one pace
        while (
            not (
                isinstance(
                    monopoly_assets.board_spaces[player.getPosition()],
                    monopoly_assets.Utility,
                )
            )
        ) and start:
            start = True
            player.updatePosition(1)
        utility = monopoly_assets.board_spaces[player.getPosition()]
        owner_info = Plugin.find_owner(utility.get_name())
        if owner_info[0]:
            if owner_info[1] == player:
                methods["send"](
                    info["address"],
                    player.getName() + " has landed on his own utility.",
                )
                Plugin.next_turn()
            else:
                methods["send"](
                    info["address"],
                    player.getName()
                    + " has landed on "
                    + owner_info[1].getName()
                    + "'s utility and must pay ten times"
                    + " whatever he rolls now with two dice!",
                )
                total = random.randint(1, 6) + random.randint(1, 6)
                methods["send"](
                    info["address"],
                    player.getName() + " rolled a total of " + str(total),
                )
                Plugin.pay(methods, info, player, owner_info[1], total * 10)
        else:
            Plugin.land_unowned_property(methods, info, player, utility)

    def move_back_three(methods, info, player):
        """
        move back three, check if new space is a property, tax space or community card
        """
        if player.getPosition == 22:
            player.setPosition(19)
            methods["send"](info["address"], "You moved back to the New York Ave. space...")
            property = monopoly_assets.board_spaces[19]
            owner_info = Plugin.find_owner(property.get_name())
            if owner_info[0]:
                if owner_info[1] == player:
                    Plugin.offer_house(methods, info, player, property)
                else:
                    rent = property.rents[property.house_count]
                    methods["send"](
                        info["address"],
                        property.get_name()
                        + " is owned by"
                        + owner_info[1].getName()
                        + " and so "
                        + player.getName()
                        + " must pay them "
                        + str(rent),
                    )
                    Plugin.pay(methods, info, player, owner_info[1], rent)
            else:
                Plugin.land_unowned_property(methods, info, player, property)

        elif player.getPosition == 26:
            methods["send"](info["address"], "You moved back to the community card space...")
            Plugin.community_card(methods, info, player)
        elif player.getPosition == 7:
            methods["send"](info["address"], "You moved back to income tax...")
            Plugin.fine(methods, info, player, 200)

    """
    PLUGIN COMMANDS
    """

    def create(methods, info, name):
        """check if game exists, otherwise create game,
        require join command rather than create command"""
        if Plugin.create_req:
            Plugin.stage = 0
            Plugin.create_req = False
            Plugin.start_join_req = True
            methods["send"](info["address"], "game has been created by " + name)
            methods["send"](info["address"], Plugin.join(name))  # automatically adds the creator
            methods["send"](
                info["address"],
                "to join this monopoly game " + "enter '.monopoly join'",
            )
        else:
            methods["send"](info["address"], "Game has already been created")

    def join(nickname):
        """add player to game if a game has been created
        and the player is not already a player"""
        if Plugin.stage == 0 and Plugin.start_join_req:
            nicknames = [player.getName() for player in Plugin.players]
            if nickname not in nicknames:
                Plugin.players.append(monopoly_assets.Player(nickname))
                return nickname + " has been added to the monopoly game"
            else:
                return nickname + " is already in the monopoly game"
        else:
            return (
                "a game has not yet started, use '.monopoly create'"
                + " to create one or '.monopoly help' to find out how this plugin works"
            )

    def start(methods, info):
        """start the game and shuffle decks"""
        if Plugin.start_join_req:
            random.shuffle(Plugin.players)
            methods["send"](
                info["address"],
                "The monopoly game has started, player order has been randomly generated:",
            )
            for player in Plugin.players:
                name = player.getName()
                methods["send"](info["address"], name)
            Plugin.get_info(methods, info)
            Plugin.turn = 0
            Plugin.stage = 1
            Plugin.roll_req = True
            Plugin.start_join_req = False
            random.shuffle(monopoly_assets.community_deck)
            random.shuffle(monopoly_assets.chance_deck)
        else:
            methods["send"](info["address"], "This command is currently not possible")

    def roll(methods, info):
        """
        roll dice to move player, unless player in prison or its already game over,
        and check new position
        """
        if Plugin.roll_req and not Plugin.game_over:
            player = Plugin.players[Plugin.turn]
            if not player.imprisoned or player.prison_time == 2:  # if user will move for sure
                roll1 = random.randint(1, 6)
                roll2 = random.randint(1, 6)
                if (
                    player.prison_time == 2 and player.imprisoned
                ):  # if this is the users last time in prison
                    methods["send"](
                        info["address"],
                        "You are in prison, this is your last "
                        + "chance to leave for free by rolling a double.",
                    )
                    player.imprisoned = False  # should free the player regardless
                    # of whether a double is thrown
                    player.prison_time = 0
                    if roll1 == roll2:
                        methods["send"](
                            info["address"],
                            "You rolled a " + str(roll1) + " twice and so are free!",
                        )
                    else:
                        methods["send"](
                            info["address"],
                            "You did not roll a double so have to pay a fee to leave!",
                        )
                        Plugin.fine(methods, info, player, 50)

                move_amount = roll1 + roll2
                passed_go = player.updatePosition(move_amount)
                new_location = monopoly_assets.board_spaces[player.getPosition()]
                methods["send"](
                    info["address"],
                    player.getName()
                    + " rolled "
                    + str(move_amount)
                    + " and is now located at "
                    + new_location.get_name()
                    + " which is board space number "
                    + str(player.getPosition()),
                )
                if (
                    isinstance(new_location, monopoly_assets.Property)
                    or isinstance(new_location, monopoly_assets.Railroad)
                ) or isinstance(new_location, monopoly_assets.Utility):
                    new_location_name = new_location.get_name()
                    new_location_owned = Plugin.find_owner(new_location_name)
                    print(new_location_owned)
                    if new_location_owned[0]:  # if property is owned by somebody
                        owner = new_location_owned[1]
                        print(owner.getName())
                        if player == owner:
                            # buy a house option
                            if isinstance(new_location, monopoly_assets.Property):
                                Plugin.offer_house(methods, info, player, new_location)
                            else:
                                methods["send"](
                                    info["address"],
                                    "You have landed on your own property "
                                    + "but you cannot buy houses for railroads or utilities",
                                )
                                Plugin.next_turn()

                        else:
                            rent = Plugin.get_rent(new_location, owner, move_amount)
                            methods["send"](
                                info["address"],
                                player.getName()
                                + " landed on a property owned by "
                                + owner.getName()
                                + " and must pay them "
                                + str(rent),
                            )
                            Plugin.pay(methods, info, player, owner, rent)

                    elif (
                        isinstance(new_location, monopoly_assets.Property)
                        or isinstance(new_location, monopoly_assets.Utility)
                        or isinstance(new_location, monopoly_assets.Railroad)
                    ):
                        # buy
                        Plugin.land_unowned_property(methods, info, player, new_location)

                elif (
                    player.getPosition() == 2
                    or player.getPosition() == 17
                    or player.getPosition() == 33
                ):
                    Plugin.community_card(methods, info, player)

                elif (
                    player.getPosition() == 7
                    or player.getPosition() == 22
                    or player.getPosition() == 36
                ):
                    Plugin.chance_card(methods, info, player)

                elif player.getPosition() == 30:
                    # check player has a get out of jail card
                    if player.get_outta_jail_card:
                        player.get_outta_jail_card = False
                        methods["send"](
                            info["address"],
                            player.getName()
                            + " landed on go to jail but used his get out of jail card...",
                        )
                        Plugin.next_turn()
                    else:
                        player.setPosition(10)
                        player.imprisoned = True
                        Plugin.next_turn()

                elif player.getPosition() == 38:
                    Plugin.fine(methods, info, player, 100)

                elif player.getPosition() == 4:
                    Plugin.fine(methods, info, player, 100)

                else:
                    Plugin.next_turn()
            else:
                methods["send"](
                    info["address"],
                    "you are in jail and will not move unless you roll a double!",
                )
                roll1 = random.randint(1, 6)
                roll2 = random.randint(1, 6)
                if roll1 == roll2:
                    player.imprisoned = False
                    player.prison_time = 0
                    move_amount = roll1 + roll2
                    passed_go = player.updatePosition(move_amount)
                    new_location = monopoly_assets.board_spaces[player.getPosition()]
                    methods["send"](
                        info["address"],
                        player.getName()
                        + " rolled "
                        + str(move_amount)
                        + " and is now located at "
                        + new_location.get_name(),
                    )
                    new_location_owned = Plugin.find_owner(new_location.get_name())
                    if new_location_owned[0]:  # if property is owned by somebody
                        owner = new_location_owned[1]
                        if player == owner:
                            # buy a house option
                            if isinstance(new_location, monopoly_assets.Property):
                                Plugin.offer_house(methods, info, player, new_location)
                            else:
                                methods["send"](
                                    info["address"],
                                    "You have landed on your own property "
                                    + "but you cannot buy houses for railroads or utilities",
                                )
                                Plugin.next_turn()
                        else:
                            rent = Plugin.get_rent(new_location, owner, move_amount)
                            methods["send"](
                                info["address"],
                                player.getName()
                                + " landed on a property owned by "
                                + owner.getName()
                                + " and must pay them "
                                + str(rent),
                            )
                            Plugin.pay(methods, info, player, owner, rent)

                    elif (
                        isinstance(new_location, monopoly_assets.Property)
                        or isinstance(new_location, monopoly_assets.Utility)
                        or isinstance(new_location, monopoly_assets.Railroad)
                    ):
                        Plugin.land_unowned_property(methods, info, player, new_location)

                    elif player.getPosition() == 22:
                        Plugin.chance_card(methods, info, player)

                    else:  # don't need to check other options as we know user starts at
                        # index ten and throwing a double limits the possibilities
                        Plugin.next_turn()

                else:
                    methods["send"](info["address"], "You did not throw a double!")
                    player.prison_time += 1
                    Plugin.next_turn()

        elif Plugin.game_over:
            methods["send"](info["address"], "The game is over!")
            methods["send"](info["address"], "The winner was " + Plugin.winner)

        else:
            methods["send"](info["address"], "Invalid command at this time")

    def leave(player_name):
        """remove player from game and change turn variable to avoid an index error"""
        if not Plugin.game_over:
            if Plugin.stage is None:
                return "no game has started yet"
            else:
                for player in Plugin.players:
                    if player.getName() == player_name:  # we should remove the player
                        if Plugin.turn == (
                            len(Plugin.players) - 1
                        ):  # check if there will be an index error when user removed
                            Plugin.turn -= 1
                        if (Plugin.players[Plugin.turn].getName()) == player_name:
                            Plugin.next_turn()
                            Plugin.roll_req = True
                            Plugin.buy_pass_req = False
                        Plugin.players.remove(player)
                        return player.getName() + " has left the game"
                        break
                else:
                    return "you are not playing in this game!"
        else:
            return "The game is already over. The winner was " + Plugin.winner

    def buy(methods, info):
        """buy property or house for already owned property"""
        if Plugin.buy_pass_req:
            Plugin.buy_pass_req = False
            Plugin.roll_req = True
            player = Plugin.players[Plugin.turn]
            property = monopoly_assets.board_spaces[player.getPosition()]
            owner_info = Plugin.find_owner(property.get_name())
            if owner_info[0]:
                if owner_info[1].getName() == player.getName():  # if owner is user
                    # buy a house
                    if int(player.getPot()) > property.house_cost:
                        if (
                            Plugin.count_player_properties(player)[property.color]
                            == monopoly_assets.Property.set_houses[property.color]
                        ):  # if full set
                            player_alive = player.reducePot(property.house_cost)
                            property.house_count += 1
                            purchase = "hotel" if property.house_count == 5 else "house"
                            methods["send"](
                                info["address"],
                                player.getName()
                                + " bought a "
                                + purchase
                                + " for "
                                + property.get_name()
                                + " so if you land there now you have to pay "
                                + str(property.rents[property.house_count]),
                            )
                            Plugin.next_turn()
                        else:
                            methods["send"](info["address"], "you need a full set to buy a house")
                            Plugin.next_turn()
                    else:
                        methods["send"](
                            info["address"],
                            "you do not have enough money to buy a house",
                        )
                        Plugin.next_turn()
            else:
                if int(player.getPot()) > property.price:
                    player_alive = player.reducePot(property.price)
                    player.portfolio.append(property)
                    if isinstance(property, monopoly_assets.Property) or isinstance(
                        property, monopoly_assets.Railroad
                    ):
                        methods["send"](
                            info["address"],
                            player.getName()
                            + " bought "
                            + property.get_name()
                            + " for "
                            + str(property.price)
                            + " and if you land there you will have to pay "
                            + str(Plugin.get_rent(property, player, 0)),
                        )
                        Plugin.next_turn()
                    elif isinstance(property, monopoly_assets.Utility):
                        if Plugin.count_player_properties(player)["utility"] == 1:
                            methods["send"](
                                info["address"],
                                player.getName()
                                + "bought "
                                + property.get_name()
                                + " for "
                                + str(property.price)
                                + " and if you land there you will have "
                                + "to pay four times whatever you roll!",
                            )
                        else:
                            methods["send"](
                                info["address"],
                                player.getName()
                                + "bought "
                                + property.get_name()
                                + " for "
                                + str(property.price)
                                + " and if you land there you will have "
                                + "to pay ten times whatever you roll!",
                            )
                else:
                    methods["send"](
                        info["address"],
                        "You do not have enough money to buy" + property.get_name(),
                    )
                    Plugin.next_turn()
        else:
            methods["send"](info["address"], "invalid command right now")

    """
    RUN PLUGIN
    """

    def run(self, incoming, methods, info, bot_info):
        try:
            if info["command"] == "PRIVMSG":
                msgs = info["args"][1:][0].split()
                if msgs[0] == ".monopoly":
                    name = info["prefix"].split("!")[0]
                    if not (
                        Plugin.checkWon(methods, info)
                    ):  # if Plugin.checkWon evaluates true it will print winner already
                        if len(msgs) == 1:
                            methods["send"](
                                info["address"],
                                ".monopoly requires an argument:"
                                + " create, join, start, roll, buy, "
                                + "pass, info, help or leave",
                            )

                        elif len(msgs) >= 3:
                            if msgs[1] == "info":
                                if msgs[2] == "rolls":
                                    player = Plugin.check_player(name)
                                    if isinstance(player, monopoly_assets.Player):
                                        Plugin.show_rolls(methods, info, player)
                                    else:
                                        methods["send"](info["address"], "You are not playing!")

                                else:
                                    pl = False  # msgs[2] not a player
                                    print(msgs[2:])
                                    for space in monopoly_assets.board_spaces:
                                        if space.get_name() == " ".join(msgs[2:]):
                                            print("space found")
                                            print(space.get_name())
                                            prop = True
                                            pl = Plugin.check_player(name)
                                            if isinstance(
                                                pl, monopoly_assets.Player
                                            ):  # check pl not false
                                                owner_info = Plugin.find_owner(space.get_name())
                                                if owner_info[0]:
                                                    if owner_info[1].getName() == pl.getName():
                                                        methods["send"](
                                                            info["address"],
                                                            "You already own " + space.get_name(),
                                                        )
                                                    else:
                                                        methods["send"](
                                                            info["address"],
                                                            space.get_name()
                                                            + " is owned by "
                                                            + owner_info[1].getName(),
                                                        )
                                                else:
                                                    methods["send"](
                                                        info["address"],
                                                        space.get_name() + " is unowned!",
                                                    )
                                                Plugin.get_location_info(methods, info, pl, space)
                                                break
                                            else:
                                                methods["send"](
                                                    info["address"],
                                                    "You are not playing and so can not "
                                                    "request this information!",
                                                )
                                    else:
                                        prop = False  # msgs[2] not a property

                                    found = False
                                    if prop is False:
                                        pl = Plugin.check_player(msgs[2])
                                        if isinstance(pl, monopoly_assets.Player):
                                            Plugin.get_player_info(methods, info, pl)
                                            found = True
                                        if not found:
                                            methods["send"](info["address"], "Invalid command")

                            else:
                                methods["send"](
                                    info["address"],
                                    "Only the info command allows"
                                    + " for a third argument which can be rolls, "
                                    + "a player name or a property name."
                                    + " Try '.monopoly info rolls'",
                                )

                        # following commands can be executed by anyone
                        elif msgs[1].lower() == "help":
                            Plugin.get_help(methods, info)

                        elif msgs[1].lower() == "create":
                            Plugin.create(methods, info, name)

                        elif msgs[1].lower() == "join":
                            methods["send"](info["address"], Plugin.join(name))

                        elif msgs[1].lower() == "info":
                            Plugin.get_info(methods, info)

                        elif msgs[1].lower() == "start":
                            Plugin.start(methods, info)

                        elif msgs[1].lower() == "leave":
                            methods["send"](info["address"], Plugin.leave(name))
                            Plugin.checkWon(methods, info)

                        elif (
                            Plugin.stage == 1
                        ):  # following commands only executable if the game has started
                            if len(Plugin.players) > 0:
                                if (
                                    name == Plugin.players[Plugin.turn].getName()
                                ):  # check the correct player issued the command

                                    if msgs[1].lower() == "roll":
                                        Plugin.roll(methods, info)

                                    elif msgs[1].lower() == "buy":
                                        Plugin.buy(methods, info)

                                    elif msgs[1].lower() == "pass":
                                        Plugin.buy_pass_req = False
                                        Plugin.roll_req = True
                                        Plugin.next_turn()

                                    else:
                                        methods["send"](
                                            info["address"],
                                            ".monopoly requires one argument:"
                                            + " create, join, start, roll, buy, "
                                            + "pass, info, help or leave",
                                        )
                                else:
                                    methods["send"](info["address"], "It is not your turn!")

                        elif (
                            msgs[1].lower() == "roll"
                            or msgs[1].lower() == "buy"
                            or msgs[1].lower() == "pass"
                        ):
                            methods["send"](
                                info["address"],
                                "Invalid command at this time, wait for game to start!",
                            )

                        else:
                            methods["send"](
                                info["address"],
                                ".monopoly requires one argument:"
                                + " create, join, start, roll, buy, "
                                + "pass, info, help or leave",
                            )

        except Exception as e:
            print("woops, monopoly plugin error ", e)



NAME = "monopoly.py"
ORIGINAL_AUTHORS = ["Angelo Giacco"]

ABOUT = """
Play monopoly
"""

COMMANDS = """
>>>.monopoly create
creates monopoly

>>>.monopoly join
user joins the game

>>>.monopoly start
starts the game

>>>.monopoly roll
rolls two dice to see how far the user will move/if the user can leave the jail

>>>.monopoly buy
user purchases property if unowned, otherwise buys house

>>>.monopoly pass
user passes on the opportunity to buy a property if unowned,
or on the opportunity to buy a house

>>>.monopoly leave
user leaves the game

>>>.monopoly help
sends messages explaining how to use the monopoly plugin

>>>.monopoly info
sends messages showing the status of each player

.monopoly info rolls
displays basic information about the upcoming spaces in front of a player

.monopoly info <<player>>
shows information about a certain player

.monopoly info <<property>>
shows information about a property
"""

WEBSITE = ""



"""
[uptime.py]
Uptime Plugin

[Author]
Nick Wiley

[About]
Returns how long the bot has been running for

[Commands]
>>> .uptime
returns the bot uptime
"""

import time


class Plugin:
    def __init__(self):
        pass

    def __convert_time(start_time):
        """
        Calculates how long the bot has been running then formats the output
        """
        current_time = time.time()
        uptime = current_time - start_time
        hrs = int(uptime / 3600)
        uptime -= hrs * 3600
        mins = int(uptime / 60)
        uptime -= mins * 60
        sec = int(uptime)

        msg = f"Bot has been up for {hrs} hr, {mins} min, and {sec} sec."
        return msg

    def run(self, incoming, methods, info, bot_info):
        try:
            if info["command"] == "PRIVMSG" and info["args"][1] == ".uptime":
                start_time = bot_info["time"]
                methods["send"](info["address"], Plugin.__convert_time(start_time))
        except Exception as e:
            print("Plugin Error: ", e)



NAME = "debug.py"
ORIGINAL_AUTHORS = ["Abdur-Rahmaan Janhangeer, pythonmembers.club"]

ABOUT = """
prints all parameters passed to bot
"""

COMMANDS = """
>>> .debug
prints some parameters
"""

WEBSITE = ""



""" game initialization """

# pylint: disable=E1601, W0612

import board
import deck
import hand
import player
import pot


def init_players(nr_of_players, starting_chips):
    """players initialization"""

    PLAYERS = [player.Player(i, starting_chips) for i in range(nr_of_players)]
    return PLAYERS


def init_game(players, round):
    """table initialization"""

    DECK = deck.Deck()
    BOARD = board.Board(DECK.make_board())
    POT = pot.Pot()
    PLAYERS = players

    for PLAYER in PLAYERS:

        PLAYER.add_hand(hand.Hand(DECK.make_hand()))

    for i in range(len(PLAYERS)):

        PLAYERS[i].add_position((len(PLAYERS) * round + (i - (round - 1))) % len(PLAYERS))

    return DECK, BOARD, POT, PLAYERS


players = init_players(6, 100)
game = init_game(players, 9)

# deck = game[0]
# board = game[1]
# pot = game[2]
players = game[3]
for card in board.get_board():

    print(card.show_card())

print(pot.show_pot())

players[0].show_player_hand().best_five(board)

for play in players:

    print(play.general_name())

    print(
        play.general_name(),
        play.show_player_hand().show_hand()[0].show_card(),
        play.show_player_hand().show_hand()[1].show_card(),
        play.chips(),
        play.position_nr(),
        play.position_name(),
        play.show_player_hand().hand_strength(board),
        str(play.show_player_hand().best_five(board)),
    )

print(len(deck.show_deck()))



"""
[Quote.py]
Quote Plugin

[Author]
Divyanshu Mehta

[About]
Extracts quotes and send random quote from Trailer Park Boys

[Commands]
>>>.quote <count>
"""
import random

import requests
from bs4 import BeautifulSoup


class Plugin:
    def __init__(self):
        pass

    def quote(self):
        data = requests.get("https://en.wikiquote.org/wiki/Trailer_Park_Boys").text
        bs = BeautifulSoup(data, "lxml")
        quotes = []
        for quote in bs.find_all("dl"):
            quotes.append(quote.text)
        return random.choice(quotes)

    def run(self, incoming, methods, info, bot_info):
        try:
            msg = info["args"][1:]
            if info["command"] == "PRIVMSG" and msg[0] == ".quote":
                for i in range(int(msg[1])):
                    methods["send"](info["address"], Plugin.quote(self))
        except Exception as e:
            print("Error:", e)



"""
[excuse.py]
Random excuse Plugin

[Author]
Rakesh Seal

[About]
Returns a random excuse for you.
[Website]


[Commands]
>>> .excuse
Returns a random excuse for you.
"""

import requests


class Plugin:
    def __init__(self):
        self.url = "https://excuser.herokuapp.com/v1/excuse"

    def run(self, incoming, methods, info, bot_info):
        try:
            msgs = info["args"][1:][0].split()
            if info["command"] == "PRIVMSG" and msgs[0] == ".excuse":
                excuse = requests.get(self.url).json()[0]["excuse"]
                methods["send"](info["address"], excuse)
        except Exception as e:
            print("No excuse for you!", e)



NAME = "send_message.py"
ORIGINAL_AUTHORS = ["Justin Walker"]

ABOUT = """
Sends a message to another channel.
"""

COMMANDS = """
>>> .send #channel .u riceabove .m how are you?
Sends a message to #channel like
'IronPenguin from #channel says to riceabove: how are you?'

>>> .send #channel hi all
Sends a message to #channel like
'IronPenguin from #channel says: hi all'

Checks if user online first and says "User not online."
"""

WEBSITE = ""



NAME = "todo.py"
ORIGINAL_AUTHORS = ["Himanshu Ranjan"]

ABOUT = """
Todo list keeps all your pending task .
You can add a task ,delete or show list.
This list is stored in a text file called todo.txt in honeybot directory.
If this does not exist it creates one.
"""

COMMANDS = """
>>>.todo add <Your Task Here>
It adds the task to do the todo list.

>>>.todo delete <index>
To delete a task input it's index number as shown in the list.

>>>.todo show
It shows all the task currently on the list.

>>>.todo clear
This will delete all the task in the list.

>>>.todo help
sends messages explaining how to use the todo plugin
"""

WEBSITE = "https://github.com/h-ranjan1110"



NAME = "Birthday Paradox"

ORIGINAL_AUTHORS = ["Paulo Ferraz"]

ABOUT = """
Show the probability that two or more people
have the same birthday in the given group.
"""

COMMANDS = """
>>> .birth
returns help on plugin use

>>> .birth <<number>>
returns the birthday matches and probability in %.
"""



NAME = "memo.py"
ORIGINAL_AUTHORS = ["Abdur-Rahmaan Janhangeer, pythonmembers.club"]

ABOUT = """
responds to .memo, demo of a basic memory plugin
"""

COMMANDS = """
>>> .memo add <key> <value>
>>> .memo rem <key>
>>> .memo fetch <key>
"""

WEBSITE = ""



"""
[random_image.py]
Random Image Plugin

[Author]
Rakesh Seal

[About]
Returns a random image url.
[Website]


[Commands]
>>> .random_image
returns a random image url.
"""

import requests


class Plugin:
    def __init__(self):
        self.url = "https://random.imagecdn.app/v1/image?width=500&height=150&category=buildings&format=json"

    def run(self, incoming, methods, info, bot_info):
        try:
            msgs = info["args"][1:][0].split()
            if info["command"] == "PRIVMSG" and msgs[0] == ".random_image":
                image_url = requests.get(self.url).json()["url"]
                methods["send"](info["address"], image_url)
        except Exception as e:
            print("blank!", e)



"""
[memo.py]
Memory Use Illustration Plugin

[Author]
Abdur-Rahmaan Janhangeer, pythonmembers.club

[About]
responds to .memo, demo of a basic memory plugin

[Commands]
>>> .memo add <key> <value>
>>> .memo rem <key>
>>> .memo fetch <key>
"""


class Plugin:
    def __init__(self):
        pass

    def run(self, incoming, methods, info, bot_info):
        try:
            msgs = info["args"][1:][0].split()
            if info["command"] == "PRIVMSG" and msgs[0] == ".memo" and msgs[1] == "add":
                methods["mem_add"]("global", "VALUES", msgs[2], msgs[3])
            if info["command"] == "PRIVMSG" and msgs[0] == ".memo" and msgs[1] == "rem":
                methods["mem_rem"]("global", "VALUES", msgs[2])
            if info["command"] == "PRIVMSG" and msgs[0] == ".memo" and msgs[1] == "fetch":
                try:
                    val = methods["mem_fetch"]("global", "VALUES", msgs[2])
                    methods["send"](info["address"], val)
                except KeyError:
                    methods["send"](info["address"], "value not found")

        except Exception as e:
            print("woops plug", e)



"""
[sysinfo.py]
System Information Plugin.

[Author]
Girish Mahabir

[About]
Responds to .sysinfo, gets System Information of the host.

[Commands]
>>> .sysinfo
returns General System Information, Boot Date and time, CPU information,
Memory information, Disk Information and Network Information.
"""
import platform
from datetime import datetime

import psutil


class Plugin:
    def __init__(self):
        pass

    def get_size(self, bytes):
        suffix = "B"
        self.bytes = bytes
        """
        Scale bytes to its proper format.
        e.g:
            1253656 => '1.20MB'
            1253656678 => '1.17GB'
        """
        factor = 1024
        for unit in ["", "K", "M", "G", "T", "P"]:
            if self.bytes < factor:
                return f"{self.bytes:.2f}{unit}{suffix}"
            self.bytes /= factor

    def run(self, incoming, methods, info, bot_info):
        try:
            if info["command"] == "PRIVMSG" and info["args"][1] == ".botinfo":
                methods["send"](info["address"], "Info Available:")
                methods["send"](info["address"], "System Information: sysinfo")
                methods["send"](info["address"], "Boot Information: bootinfo")
                methods["send"](info["address"], "CPU Information: cpuinfo")
                methods["send"](info["address"], "Memory Information: memoryinfo")
                methods["send"](info["address"], "Disk Information: diskinfo")
                methods["send"](info["address"], "Network Information: networkinfo")

            if info["command"] == "PRIVMSG" and info["args"][1] == ".sysinfo":
                # System Information.
                methods["send"](info["address"], "=" * 40 + "System Information" + "=" * 40)
                uname = platform.uname()
                methods["send"](info["address"], f"System: {uname.system}")
                methods["send"](info["address"], f"Node Name: {uname.node}")
                methods["send"](info["address"], f"Release: {uname.release}")
                methods["send"](info["address"], f"Version: {uname.version}")
                methods["send"](info["address"], f"Machine: {uname.machine}")
                methods["send"](info["address"], f"Processor: {uname.processor}")

            if info["command"] == "PRIVMSG" and info["args"][1] == ".bootinfo":
                # Data and Time computer was booted.
                # Boot Time.
                methods["send"](info["address"], "=" * 40 + "Boot Time" + "=" * 40)
                boot_time_timestamp = psutil.boot_time()
                bt = datetime.fromtimestamp(boot_time_timestamp)
                methods["send"](
                    info["address"],
                    f"Boot Time: {bt.year}/{bt.month}/{bt.day} {bt.hour}:{bt.minute}:{bt.second}",
                )

            if info["command"] == "PRIVMSG" and info["args"][1] == ".cpuinfo":
                # CPU Information.
                # Let's Print, CPU information.
                methods["send"](info["address"], "=" * 40 + "CPU Info" + "=" * 40)
                # Number of cores.
                methods["send"](
                    info["address"],
                    f"Physical cores: {psutil.cpu_count(logical=False)}",
                )
                methods["send"](
                    info["address"],
                    f"Total cores: {psutil.cpu_count(logical=True)}",
                )
                # CPU frequencies.
                cpufreq = psutil.cpu_freq()
                methods["send"](info["address"], f"Max Frequency: {cpufreq.max:.2f}Mhz")
                methods["send"](info["address"], f"Min Frequency: {cpufreq.min:.2f}Mhz")
                methods["send"](info["address"], f"Current Frequency: {cpufreq.current:.2f}Mhz")
                # CPU usage.
                methods["send"](info["address"], "CPU Usage Per Core:")
                for i, percentage in enumerate(psutil.cpu_percent(percpu=True)):
                    methods["send"](info["address"], f"Core {i}: {percentage}%")
                methods["send"](info["address"], f"Total CPU Usage: {psutil.cpu_percent()}%")

            if info["command"] == "PRIVMSG" and info["args"][1] == ".memoryinfo":
                # Memory Information.
                methods["send"](info["address"], "=" * 40 + "Memory Information" + "=" * 40)
                # Get the memory details.
                svmem = psutil.virtual_memory()
                methods["send"](info["address"], f"Total: {self.get_size(svmem.total)}")
                methods["send"](info["address"], f"Available: {self.get_size(svmem.available)}")
                methods["send"](info["address"], f"Used: {self.get_size(svmem.used)}")
                methods["send"](info["address"], f"Percentage: {svmem.percent}%")
                methods["send"](info["address"], "=" * 20 + "SWAP" + "=" * 20)
                # Get the swap memory details (if exists).
                swap = psutil.swap_memory()
                methods["send"](info["address"], f"Total: {self.get_size(swap.total)}")
                methods["send"](info["address"], f"Free: {self.get_size(swap.free)}")
                methods["send"](info["address"], f"Used: {self.get_size(swap.used)}")
                methods["send"](info["address"], f"Percentage: {swap.percent}%")

            if info["command"] == "PRIVMSG" and info["args"][1] == ".networkinfo":
                # Network information
                methods["send"](info["address"], "=" * 40 + "Network Information" + "=" * 40)
                # Get all network interfaces (virtual and physical).
                if_addrs = psutil.net_if_addrs()
                for interface_name, interface_addresses in if_addrs.items():
                    for address in interface_addresses:
                        # methods["send"](info["address"],
                        # f"=== Interface: {interface_name} ===")
                        if str(address.family) == "AddressFamily.AF_INET":
                            methods["send"](
                                info["address"],
                                f"IP Address: {address.address}  ||  "
                                f"Netmask: {address.netmask}".format(),
                            )
                        elif str(address.family) == "AddressFamily.AF_PACKET":
                            methods["send"](
                                info["address"],
                                f"MAC Address: {address.address} ||  Netmask: {address.netmask}",
                            )
                # Get IO statistics since boot.
                net_io = psutil.net_io_counters()
                methods["send"](
                    info["address"],
                    f"Total Bytes Sent: {self.get_size(net_io.bytes_sent)}"
                    " ||  Total Bytes Received: "
                    f"{self.get_size(net_io.bytes_recv)}",
                )

            if info["command"] == "PRIVMSG" and info["args"][1] == ".diskinfo":
                # Disk Information
                methods["send"](
                    info["address"],
                    "=" * 40 + "Disk Information" + "=" * 40 + "\nPartitions and Usage:",
                )
                # Get all disk partitions.
                partitions = psutil.disk_partitions()
                for partition in partitions:
                    methods["send"](
                        info["address"],
                        f"=== Device: {partition.device} ===\n"
                        f"  Mountpoint: {partition.mountpoint}\n"
                        f"  File system type: {partition.fstype}",
                    )
                    try:
                        partition_usage = psutil.disk_usage(partition.mountpoint)
                    except PermissionError:
                        # This can be catch due to disks that
                        # isn't ready.
                        continue
                    methods["send"](
                        info["address"],
                        "  Total Size: "
                        f"{self.get_size(partition_usage.total)}\n  "
                        f"Used: {self.get_size(partition_usage.used)}\n  "
                        f"Free: {self.get_size(partition_usage.free)}\n  "
                        f"Percentage: {partition_usage.percent}%",
                    )
                # Get IO statistics since boot.
                disk_io = psutil.disk_io_counters()
                methods["send"](
                    info["address"],
                    f"Total read: {self.get_size(disk_io.read_bytes)}\n"
                    f"Total write: {self.get_size(disk_io.write_bytes)}",
                )
        except Exception as e:
            print("woops plug", e)



"""
[help.py]
Help Plugin

[Author]
Eduardo Moraes de Mello, https://github.com/edumello

[About]
Show link with all the functionalities of the bot.

[Commands]
>>> .help
returns link of plugins informations
"""


class Plugin:
    def __init__(self):
        pass

    def run(self, incoming, methods, info, bot_info):

        try:
            if info["command"] == "PRIVMSG" and info["args"][1] == ".help":
                message = (
                    "Page with all functionalities and commands: "
                    "https://github.com/Abdur-rahmaanJ/"
                    "honeybot/blob/master/honeybot/plugins_info.md"
                )
                methods["send"](info["address"], message)
        except Exception as e:
            print("woops plug", e)



"""
[countries.py]
Country-Code Plugin

[Author]
Sarvesh Dubey

[About]
Show the dialcode of a given country.

[Commands]
>>> .tel <country_code>
returns the associated country telephone code
"""


class Plugin:
    def __init__(self):
        pass

    def __tel(input_country):

        d = [
            {"name": "Afghanistan", "dial_code": "+93", "code": "AF"},
            {"name": "Aland Islands", "dial_code": "+358", "code": "AX"},
            {"name": "Albania", "dial_code": "+355", "code": "AL"},
            {"name": "Algeria", "dial_code": "+213", "code": "DZ"},
            {"name": "AmericanSamoa", "dial_code": "+1 684", "code": "AS"},
            {"name": "Andorra", "dial_code": "+376", "code": "AD"},
            {"name": "Angola", "dial_code": "+244", "code": "AO"},
            {"name": "Anguilla", "dial_code": "+1 264", "code": "AI"},
            {"name": "Antarctica", "dial_code": "+672", "code": "AQ"},
            {"name": "Antigua and Barbuda", "dial_code": "+1268", "code": "AG"},
            {"name": "Argentina", "dial_code": "+54", "code": "AR"},
            {"name": "Armenia", "dial_code": "+374", "code": "AM"},
            {"name": "Aruba", "dial_code": "+297", "code": "AW"},
            {"name": "Australia", "dial_code": "+61", "code": "AU"},
            {"name": "Austria", "dial_code": "+43", "code": "AT"},
            {"name": "Azerbaijan", "dial_code": "+994", "code": "AZ"},
            {"name": "Bahamas", "dial_code": "+1 242", "code": "BS"},
            {"name": "Bahrain", "dial_code": "+973", "code": "BH"},
            {"name": "Bangladesh", "dial_code": "+880", "code": "BD"},
            {"name": "Barbados", "dial_code": "+1 246", "code": "BB"},
            {"name": "Belarus", "dial_code": "+375", "code": "BY"},
            {"name": "Belgium", "dial_code": "+32", "code": "BE"},
            {"name": "Belize", "dial_code": "+501", "code": "BZ"},
            {"name": "Benin", "dial_code": "+229", "code": "BJ"},
            {"name": "Bermuda", "dial_code": "+1 441", "code": "BM"},
            {"name": "Bhutan", "dial_code": "+975", "code": "BT"},
            {
                "name": "Bolivia, Plurinational State of",
                "dial_code": "+591",
                "code": "BO",
            },
            {"name": "Bosnia and Herzegovina", "dial_code": "+387", "code": "BA"},
            {"name": "Botswana", "dial_code": "+267", "code": "BW"},
            {"name": "Brazil", "dial_code": "+55", "code": "BR"},
            {
                "name": "British Indian Ocean Territory",
                "dial_code": "+246",
                "code": "IO",
            },
            {"name": "Brunei Darussalam", "dial_code": "+673", "code": "BN"},
            {"name": "Bulgaria", "dial_code": "+359", "code": "BG"},
            {"name": "Burkina Faso", "dial_code": "+226", "code": "BF"},
            {"name": "Burundi", "dial_code": "+257", "code": "BI"},
            {"name": "Cambodia", "dial_code": "+855", "code": "KH"},
            {"name": "Cameroon", "dial_code": "+237", "code": "CM"},
            {"name": "Canada", "dial_code": "+1", "code": "CA"},
            {"name": "Cape Verde", "dial_code": "+238", "code": "CV"},
            {"name": "Cayman Islands", "dial_code": "+ 345", "code": "KY"},
            {"name": "Central African Republic", "dial_code": "+236", "code": "CF"},
            {"name": "Chad", "dial_code": "+235", "code": "TD"},
            {"name": "Chile", "dial_code": "+56", "code": "CL"},
            {"name": "China", "dial_code": "+86", "code": "CN"},
            {"name": "Christmas Island", "dial_code": "+61", "code": "CX"},
            {"name": "Cocos (Keeling) Islands", "dial_code": "+61", "code": "CC"},
            {"name": "Colombia", "dial_code": "+57", "code": "CO"},
            {"name": "Comoros", "dial_code": "+269", "code": "KM"},
            {"name": "Congo", "dial_code": "+242", "code": "CG"},
            {
                "name": "Congo, The Democratic Republic of the Congo",
                "dial_code": "+243",
                "code": "CD",
            },
            {"name": "Cook Islands", "dial_code": "+682", "code": "CK"},
            {"name": "Costa Rica", "dial_code": "+506", "code": "CR"},
            {"name": "Cote d'Ivoire", "dial_code": "+225", "code": "CI"},
            {"name": "Croatia", "dial_code": "+385", "code": "HR"},
            {"name": "Cuba", "dial_code": "+53", "code": "CU"},
            {"name": "Cyprus", "dial_code": "+357", "code": "CY"},
            {"name": "Czech Republic", "dial_code": "+420", "code": "CZ"},
            {"name": "Denmark", "dial_code": "+45", "code": "DK"},
            {"name": "Djibouti", "dial_code": "+253", "code": "DJ"},
            {"name": "Dominica", "dial_code": "+1 767", "code": "DM"},
            {"name": "Dominican Republic", "dial_code": "+1 849", "code": "DO"},
            {"name": "Ecuador", "dial_code": "+593", "code": "EC"},
            {"name": "Egypt", "dial_code": "+20", "code": "EG"},
            {"name": "El Salvador", "dial_code": "+503", "code": "SV"},
            {"name": "Equatorial Guinea", "dial_code": "+240", "code": "GQ"},
            {"name": "Eritrea", "dial_code": "+291", "code": "ER"},
            {"name": "Estonia", "dial_code": "+372", "code": "EE"},
            {"name": "Ethiopia", "dial_code": "+251", "code": "ET"},
            {"name": "Falkland Islands (Malvinas)", "dial_code": "+500", "code": "FK"},
            {"name": "Faroe Islands", "dial_code": "+298", "code": "FO"},
            {"name": "Fiji", "dial_code": "+679", "code": "FJ"},
            {"name": "Finland", "dial_code": "+358", "code": "FI"},
            {"name": "France", "dial_code": "+33", "code": "FR"},
            {"name": "French Guiana", "dial_code": "+594", "code": "GF"},
            {"name": "French Polynesia", "dial_code": "+689", "code": "PF"},
            {"name": "Gabon", "dial_code": "+241", "code": "GA"},
            {"name": "Gambia", "dial_code": "+220", "code": "GM"},
            {"name": "Georgia", "dial_code": "+995", "code": "GE"},
            {"name": "Germany", "dial_code": "+49", "code": "DE"},
            {"name": "Ghana", "dial_code": "+233", "code": "GH"},
            {"name": "Gibraltar", "dial_code": "+350", "code": "GI"},
            {"name": "Greece", "dial_code": "+30", "code": "GR"},
            {"name": "Greenland", "dial_code": "+299", "code": "GL"},
            {"name": "Grenada", "dial_code": "+1 473", "code": "GD"},
            {"name": "Guadeloupe", "dial_code": "+590", "code": "GP"},
            {"name": "Guam", "dial_code": "+1 671", "code": "GU"},
            {"name": "Guatemala", "dial_code": "+502", "code": "GT"},
            {"name": "Guernsey", "dial_code": "+44", "code": "GG"},
            {"name": "Guinea", "dial_code": "+224", "code": "GN"},
            {"name": "Guinea-Bissau", "dial_code": "+245", "code": "GW"},
            {"name": "Guyana", "dial_code": "+595", "code": "GY"},
            {"name": "Haiti", "dial_code": "+509", "code": "HT"},
            {
                "name": "Holy See (Vatican City State)",
                "dial_code": "+379",
                "code": "VA",
            },
            {"name": "Honduras", "dial_code": "+504", "code": "HN"},
            {"name": "Hong Kong", "dial_code": "+852", "code": "HK"},
            {"name": "Hungary", "dial_code": "+36", "code": "HU"},
            {"name": "Iceland", "dial_code": "+354", "code": "IS"},
            {"name": "India", "dial_code": "+91", "code": "IN"},
            {"name": "Indonesia", "dial_code": "+62", "code": "ID"},
            {
                "name": "Iran, Islamic Republic of Persian Gulf",
                "dial_code": "+98",
                "code": "IR",
            },
            {"name": "Iraq", "dial_code": "+964", "code": "IQ"},
            {"name": "Ireland", "dial_code": "+353", "code": "IE"},
            {"name": "Isle of Man", "dial_code": "+44", "code": "IM"},
            {"name": "Israel", "dial_code": "+972", "code": "IL"},
            {"name": "Italy", "dial_code": "+39", "code": "IT"},
            {"name": "Jamaica", "dial_code": "+1 876", "code": "JM"},
            {"name": "Japan", "dial_code": "+81", "code": "JP"},
            {"name": "Jersey", "dial_code": "+44", "code": "JE"},
            {"name": "Jordan", "dial_code": "+962", "code": "JO"},
            {"name": "Kazakhstan", "dial_code": "+7 7", "code": "KZ"},
            {"name": "Kenya", "dial_code": "+254", "code": "KE"},
            {"name": "Kiribati", "dial_code": "+686", "code": "KI"},
            {
                "name": "Korea, Democratic People's Republic of Korea",
                "dial_code": "+850",
                "code": "KP",
            },
            {
                "name": "Korea, Republic of South Korea",
                "dial_code": "+82",
                "code": "KR",
            },
            {"name": "Kuwait", "dial_code": "+965", "code": "KW"},
            {"name": "Kyrgyzstan", "dial_code": "+996", "code": "KG"},
            {"name": "Laos", "dial_code": "+856", "code": "LA"},
            {"name": "Latvia", "dial_code": "+371", "code": "LV"},
            {"name": "Lebanon", "dial_code": "+961", "code": "LB"},
            {"name": "Lesotho", "dial_code": "+266", "code": "LS"},
            {"name": "Liberia", "dial_code": "+231", "code": "LR"},
            {"name": "Libyan Arab Jamahiriya", "dial_code": "+218", "code": "LY"},
            {"name": "Liechtenstein", "dial_code": "+423", "code": "LI"},
            {"name": "Lithuania", "dial_code": "+370", "code": "LT"},
            {"name": "Luxembourg", "dial_code": "+352", "code": "LU"},
            {"name": "Macao", "dial_code": "+853", "code": "MO"},
            {"name": "Macedonia", "dial_code": "+389", "code": "MK"},
            {"name": "Madagascar", "dial_code": "+261", "code": "MG"},
            {"name": "Malawi", "dial_code": "+265", "code": "MW"},
            {"name": "Malaysia", "dial_code": "+60", "code": "MY"},
            {"name": "Maldives", "dial_code": "+960", "code": "MV"},
            {"name": "Mali", "dial_code": "+223", "code": "ML"},
            {"name": "Malta", "dial_code": "+356", "code": "MT"},
            {"name": "Marshall Islands", "dial_code": "+692", "code": "MH"},
            {"name": "Martinique", "dial_code": "+596", "code": "MQ"},
            {"name": "Mauritania", "dial_code": "+222", "code": "MR"},
            {"name": "Mauritius", "dial_code": "+230", "code": "MU"},
            {"name": "Mayotte", "dial_code": "+262", "code": "YT"},
            {"name": "Mexico", "dial_code": "+52", "code": "MX"},
            {
                "name": "Micronesia, Federated States of Micronesia",
                "dial_code": "+691",
                "code": "FM",
            },
            {"name": "Moldova", "dial_code": "+373", "code": "MD"},
            {"name": "Monaco", "dial_code": "+377", "code": "MC"},
            {"name": "Mongolia", "dial_code": "+976", "code": "MN"},
            {"name": "Montenegro", "dial_code": "+382", "code": "ME"},
            {"name": "Montserrat", "dial_code": "+1664", "code": "MS"},
            {"name": "Morocco", "dial_code": "+212", "code": "MA"},
            {"name": "Mozambique", "dial_code": "+258", "code": "MZ"},
            {"name": "Myanmar", "dial_code": "+95", "code": "MM"},
            {"name": "Namibia", "dial_code": "+264", "code": "NA"},
            {"name": "Nauru", "dial_code": "+674", "code": "NR"},
            {"name": "Nepal", "dial_code": "+977", "code": "NP"},
            {"name": "Netherlands", "dial_code": "+31", "code": "NL"},
            {"name": "Netherlands Antilles", "dial_code": "+599", "code": "AN"},
            {"name": "New Caledonia", "dial_code": "+687", "code": "NC"},
            {"name": "New Zealand", "dial_code": "+64", "code": "NZ"},
            {"name": "Nicaragua", "dial_code": "+505", "code": "NI"},
            {"name": "Niger", "dial_code": "+227", "code": "NE"},
            {"name": "Nigeria", "dial_code": "+234", "code": "NG"},
            {"name": "Niue", "dial_code": "+683", "code": "NU"},
            {"name": "Norfolk Island", "dial_code": "+672", "code": "NF"},
            {"name": "Northern Mariana Islands", "dial_code": "+1 670", "code": "MP"},
            {"name": "Norway", "dial_code": "+47", "code": "NO"},
            {"name": "Oman", "dial_code": "+968", "code": "OM"},
            {"name": "Pakistan", "dial_code": "+92", "code": "PK"},
            {"name": "Palau", "dial_code": "+680", "code": "PW"},
            {
                "name": "Palestinian Territory, Occupied",
                "dial_code": "+970",
                "code": "PS",
            },
            {"name": "Panama", "dial_code": "+507", "code": "PA"},
            {"name": "Papua New Guinea", "dial_code": "+675", "code": "PG"},
            {"name": "Paraguay", "dial_code": "+595", "code": "PY"},
            {"name": "Peru", "dial_code": "+51", "code": "PE"},
            {"name": "Philippines", "dial_code": "+63", "code": "PH"},
            {"name": "Pitcairn", "dial_code": "+872", "code": "PN"},
            {"name": "Poland", "dial_code": "+48", "code": "PL"},
            {"name": "Portugal", "dial_code": "+351", "code": "PT"},
            {"name": "Puerto Rico", "dial_code": "+1 939", "code": "PR"},
            {"name": "Qatar", "dial_code": "+974", "code": "QA"},
            {"name": "Romania", "dial_code": "+40", "code": "RO"},
            {"name": "Russia", "dial_code": "+7", "code": "RU"},
            {"name": "Rwanda", "dial_code": "+250", "code": "RW"},
            {"name": "Reunion", "dial_code": "+262", "code": "RE"},
            {"name": "Saint Barthelemy", "dial_code": "+590", "code": "BL"},
            {
                "name": "Saint Helena, Ascension and Tristan Da Cunha",
                "dial_code": "+290",
                "code": "SH",
            },
            {"name": "Saint Kitts and Nevis", "dial_code": "+1 869", "code": "KN"},
            {"name": "Saint Lucia", "dial_code": "+1 758", "code": "LC"},
            {"name": "Saint Martin", "dial_code": "+590", "code": "MF"},
            {"name": "Saint Pierre and Miquelon", "dial_code": "+508", "code": "PM"},
            {
                "name": "Saint Vincent and the Grenadines",
                "dial_code": "+1 784",
                "code": "VC",
            },
            {"name": "Samoa", "dial_code": "+685", "code": "WS"},
            {"name": "San Marino", "dial_code": "+378", "code": "SM"},
            {"name": "Sao Tome and Principe", "dial_code": "+239", "code": "ST"},
            {"name": "Saudi Arabia", "dial_code": "+966", "code": "SA"},
            {"name": "Senegal", "dial_code": "+221", "code": "SN"},
            {"name": "Serbia", "dial_code": "+381", "code": "RS"},
            {"name": "Seychelles", "dial_code": "+248", "code": "SC"},
            {"name": "Sierra Leone", "dial_code": "+232", "code": "SL"},
            {"name": "Singapore", "dial_code": "+65", "code": "SG"},
            {"name": "Slovakia", "dial_code": "+421", "code": "SK"},
            {"name": "Slovenia", "dial_code": "+386", "code": "SI"},
            {"name": "Solomon Islands", "dial_code": "+677", "code": "SB"},
            {"name": "Somalia", "dial_code": "+252", "code": "SO"},
            {"name": "South Africa", "dial_code": "+27", "code": "ZA"},
            {
                "name": "South Georgia and the South Sandwich Islands",
                "dial_code": "+500",
                "code": "GS",
            },
            {"name": "Spain", "dial_code": "+34", "code": "ES"},
            {"name": "Sri Lanka", "dial_code": "+94", "code": "LK"},
            {"name": "Sudan", "dial_code": "+249", "code": "SD"},
            {"name": "Suriname", "dial_code": "+597", "code": "SR"},
            {"name": "Svalbard and Jan Mayen", "dial_code": "+47", "code": "SJ"},
            {"name": "Swaziland", "dial_code": "+268", "code": "SZ"},
            {"name": "Sweden", "dial_code": "+46", "code": "SE"},
            {"name": "Switzerland", "dial_code": "+41", "code": "CH"},
            {"name": "Syrian Arab Republic", "dial_code": "+963", "code": "SY"},
            {"name": "Taiwan", "dial_code": "+886", "code": "TW"},
            {"name": "Tajikistan", "dial_code": "+992", "code": "TJ"},
            {
                "name": "Tanzania, United Republic of Tanzania",
                "dial_code": "+255",
                "code": "TZ",
            },
            {"name": "Thailand", "dial_code": "+66", "code": "TH"},
            {"name": "Timor-Leste", "dial_code": "+670", "code": "TL"},
            {"name": "Togo", "dial_code": "+228", "code": "TG"},
            {"name": "Tokelau", "dial_code": "+690", "code": "TK"},
            {"name": "Tonga", "dial_code": "+676", "code": "TO"},
            {"name": "Trinidad and Tobago", "dial_code": "+1 868", "code": "TT"},
            {"name": "Tunisia", "dial_code": "+216", "code": "TN"},
            {"name": "Turkey", "dial_code": "+90", "code": "TR"},
            {"name": "Turkmenistan", "dial_code": "+993", "code": "TM"},
            {"name": "Turks and Caicos Islands", "dial_code": "+1 649", "code": "TC"},
            {"name": "Tuvalu", "dial_code": "+688", "code": "TV"},
            {"name": "Uganda", "dial_code": "+256", "code": "UG"},
            {"name": "Ukraine", "dial_code": "+380", "code": "UA"},
            {"name": "United Arab Emirates", "dial_code": "+971", "code": "AE"},
            {"name": "United Kingdom", "dial_code": "+44", "code": "GB"},
            {"name": "United States", "dial_code": "+1", "code": "US"},
            {"name": "Uruguay", "dial_code": "+598", "code": "UY"},
            {"name": "Uzbekistan", "dial_code": "+998", "code": "UZ"},
            {"name": "Vanuatu", "dial_code": "+678", "code": "VU"},
            {
                "name": "Venezuela, Bolivarian Republic of Venezuela",
                "dial_code": "+58",
                "code": "VE",
            },
            {"name": "Vietnam", "dial_code": "+84", "code": "VN"},
            {"name": "Virgin Islands, British", "dial_code": "+1 284", "code": "VG"},
            {"name": "Virgin Islands, U.S.", "dial_code": "+1 340", "code": "VI"},
            {"name": "Wallis and Futuna", "dial_code": "+681", "code": "WF"},
            {"name": "Yemen", "dial_code": "+967", "code": "YE"},
            {"name": "Zambia", "dial_code": "+260", "code": "ZM"},
            {"name": "Zimbabwe", "dial_code": "+263", "code": "ZW"},
        ]
        country_code = input_country.upper()
        for i in range(len(d)):
            s = d[i]
            if s["code"] == country_code:
                return s["dial_code"]

    def run(self, incoming, methods, info, bot_info):
        try:
            msgs = info["args"]
            input_country = str(msgs[1][5:])

            if info["command"] == "PRIVMSG" and info["args"][1][1:4] == "tel":
                methods["send"](info["address"], Plugin.__tel(input_country))
        except Exception as e:
            print("woops plugin error: ", e)



"""
[calc.py]
Caesar Cipher plugin

[Author]
Kyle Galloway

[About]
encrypts and decrypts using a caesar cipher

[Commands]
>>> .caesar_cipher <message>
returns the encrypted version of message
"""
import codecs


class Plugin:
    def __init__(self):
        pass

    def run(self, incoming, methods, info, bot_info):
        try:
            msgs = info["args"][1:][0].split()

            if info["command"] == "PRIVMSG":
                if len(msgs) > 1:
                    if msgs[0] == ".caesar_cipher":
                        expr = msgs[1]
                        encoded = codecs.encode(expr, "rot-13")
                        print(encoded)
                        methods["send"](info["address"], f"{encoded}")
        except Exception as e:
            print("woops plugin", __file__, e)



"""
[todo.py]
Todo list Plugin

[Author]
Himanshu Ranjan

[Website]
https://github.com/h-ranjan1110

[About]
Todo list keeps all your pending task .
You can add a task ,delete or show list.
This list is stored in a text file called todo.txt in honeybot directory.
If this does not exist it creates one.

[Commands]
>>>.todo add <Your Task Here>
It adds the task to do the todo list.

>>>.todo delete <index>
To delete a task input it's index number as shown in the list.

>>>.todo show
It shows all the task currently on the list.

>>>.todo clear
This will delete all the task in the list.

>>>.todo help
sends messages explaining how to use the todo plugin
"""

import os

if not os.path.exists("todo.txt"):
    """Creating todo.txt is it is not already present."""
    todo = open("todo.txt", "w+")
    todo.close()


class Plugin:
    def __init__(self):
        pass

    def get_help(info, methods):
        """help function to give information about commands."""
        methods["send"](
            info["address"],
            ".todo add  <Your Task Here>             (It adds the task to do the todo list.)",
        )
        methods["send"](
            info["address"],
            ".todo delete  <index>             (To delete a task input it's index number as shown"
            " in the list.)",
        )
        methods["send"](
            info["address"],
            ".todo show             (It shows all the task currently on the list.)",
        )
        methods["send"](
            info["address"],
            ".todo clear             (This will delete all the task in the list.)",
        )
        methods["send"](
            info["address"],
            ".todo help             (sends messages explaining how to use the todo plugin.)",
        )

    def showlist(info, methods):
        """It prints the todo file on the screen"""
        # Parse the user ID from info['prefix']
        raw_user = info["prefix"]
        user_index = raw_user.find("!")
        user = raw_user[0:user_index]

        if os.stat("todo.txt").st_size == 0:
            methods["send"](info["address"], "Awesome " + user + ". You Have No Task Pending!")
        else:
            methods["send"](info["address"], "Hi, " + user + ". These are your tasks.")
            with open("todo.txt") as f:
                lines = f.readlines()
            count = 1
            for line in lines:
                methods["send"](info["address"], str(count) + ". " + line)
                count = count + 1

    def run(self, incoming, methods, info, bot_info):
        """Handling of requests from user."""
        try:
            msgs = info["args"][1:][0].split()

            if info["command"] == "PRIVMSG" and msgs[0] == ".todo":

                # adding new task to beginning of list
                if msgs[1] == "add":
                    task_start_index = 9
                    task = info["args"][1:][0][task_start_index:]
                    with open("todo.txt", "r+") as todo:
                        content = todo.read()
                        todo.seek(0, 0)
                        todo.write(task.rstrip("\n") + "\n" + content)
                    Plugin.showlist(info, methods)

                # deleting the line of given index
                elif msgs[1] == "delete":
                    line_no = int(msgs[2])

                    with open("todo.txt") as f:
                        lines = f.readlines()

                    count = 1
                    with open("todo.txt", "w") as f:
                        for line in lines:
                            if count != line_no:
                                f.write(line)
                            count = count + 1
                    Plugin.showlist(info, methods)

                # Deleting all the task . todo.txt file is not deleted
                elif msgs[1] == "clear":
                    open("todo.txt", "w").close()
                    Plugin.showlist(info, methods)

                # Displaying all task in file .
                elif msgs[1] == "show":
                    Plugin.showlist(info, methods)

                # Showing help
                elif msgs[1] == "help":
                    Plugin.get_help(info, methods)
                else:
                    methods["send"](info["address"], "Please Select A Valid Option")

        except Exception as e:
            print("woops plugin", __file__, e)



NAME = "fact.py"
ORIGINAL_AUTHORS = ["Justin Walker"]

ABOUT = """
Returns a random fact.
Facts sourced from Mental Floss
http://mentalfloss.com/article/55443/101-amazing-facts
"""

COMMANDS = """
>>> .fact
returns random fact
"""

WEBSITE = ""



"""
[installed_module.py]
Installation checker

[Author]
Abdur-Rahmaan Janhangeer, pythonmembers.club

[About]
checks if all listed in requirements.txt installed

[Commands]
>>> .installed
"""

import importlib


class Plugin:
    def __init__(self):
        pass

    def run(self, incoming, methods, info, bot_info):
        try:
            # if '!~' in info['prefix']:
            # print(info)
            if info["command"] == "PRIVMSG" and info["args"][1] == ".installed":
                reqs = bot_info["required_modules"]
                not_found = []
                for module in reqs:
                    try:
                        importlib.import_module(module)
                    except ModuleNotFoundError as e:
                        print("not found:", module)
                        not_found.append(module)
                if not_found:
                    methods["send"](info["address"], "not found:{}".format("-".join(not_found)))
                else:
                    methods["send"](info["address"], "all required modules installed")
        except Exception as e:
            print("woops plug", e)



""" board class """

# pylint: disable=E1601


class Board:
    """board class"""

    def __init__(self, board):
        """board initialization"""

        self.__board = board
        self.__flop = board[:3]
        self.__flop1 = board[0]
        self.__flop2 = board[1]
        self.__flop3 = board[2]
        self.__turn = board[3]
        self.__river = board[4]

    def get_board(self):
        """return card objects of board"""
        return self.__board

    def show_board(self):
        """show board"""
        return " ".join([c.show_card() for c in self.__board])

    def flop(self):
        """show flop"""

        return " ".join([c.show_card() for c in self.__flop])

    def flop1(self):
        """show flop1"""

        return self.__flop1.show_card()

    def flop2(self):
        """show flop2"""

        return self.__flop2.show_card()

    def flop3(self):
        """show flop3"""

        return self.__flop3.show_card()

    def turn(self):
        """show turn"""

        return self.__turn.show_card()

    def river(self):
        """show river"""

        return self.__river.show_card()

    def __len__(self):
        return len(self.__board)



"""
[random_color.py]
Random Color Plugin

[Author]
Jeet Trivedi

[About]
responds to .randcol, returns a HEX color code

[Commands]
>>> .randcol
returns a HEX color code: a 6-character string representing RGB
"""

from random import randrange


class Plugin:
    def __init__(self):
        pass

    # this function returns a random 6-character HEX string, representing RGB
    def __gen_rgb_hex():
        rgb_hex_chars = [hex(randrange(16))[2:] for hex_char in range(6)]
        rgb_hex_str = "".join(rgb_hex_chars)
        rgb_hex_str = rgb_hex_str.upper()
        return rgb_hex_str

    def run(self, incoming, methods, info, bot_info):
        try:
            # if '!~' in info['prefix']:
            # print(info)
            if info["command"] == "PRIVMSG" and info["args"][1] == ".randcol":
                methods["send"](info["address"], "Hex: " + Plugin.__gen_rgb_hex())
        except Exception as e:
            print("woops plug", e)



"""
[movies_imdb.py]
IMDb Movies Plugin

[Author]
Zoi Katsantoni

[About]
Searches a movie then returns certain information

[Commands]

> > > .movies_imdb <<search>> <<movie>> <<information>>
> > > returns the certain information about the movie
> > > .movies_imdb 250movies
> > > returns 250 all-time most popular movies
> > > """

import imdb


class Plugin:
    movies = imdb.IMDb()

    def __init__(self):
        pass

    def search_movie_by_name(movie):
        movies = imdb.IMDb()
        certain_movie = movies.search_movie(movie)
        if not certain_movie:
            print("Could not find this movie")
            return -1
        else:
            ret = certain_movie[0]
            return ret.movieID

    def get_movie_from_id(movie_id):
        movies = imdb.IMDb()
        return movies.get_movie(movie_id)

    def get_movie_cast(movie):
        to_return = "Title: " + str(movie["title"])
        for cast in movie["cast"]:
            to_return += cast["name"] + " " + cast.currentRole + "\n"
        return to_return

    def get_movie_directors(movie):
        to_return = "Title: " + str(movie["title"]) + "\n" + "Directors: \n"
        for director in movie["director"]:
            to_return += director + "\n"
        return to_return

    def get_movie_producers(movie):
        to_return = "Title: " + str(movie["title"]) + "\n" + "Producers: \n"
        for producer in movie["producer"]:
            to_return += producer + "\n"
        return to_return

    def get_movie_rating(movie):
        return str(movie["rating"])

    def get_movie_year(movie):
        return str(movie["year"])

    def get_movie_genre(movie):
        to_return = ""
        for genre in movie["genres"]:
            to_return += genre + "\n"
        return to_return

    def get_movie_runtime(movie):
        return str(movie["runtimes"][0] + " minutes")

    def get_movie_countries(movie):
        to_return = ""
        for country in movie["countries"]:
            to_return += country
        return to_return

    def top_movies(movie2):
        movies = imdb.IMDb()
        top = movies.get_top250_movies()
        i = 1
        all_together = ""
        for movie in top:
            all_together += str(i) + ". " + movie["title"]
            i += 1
        return all_together

    def run(self, incoming, methods, info, bot_info):
        try:
            movies = imdb.IMDb()
            movie2 = ""
            msgs = info["args"][1:][0].split()

            if msgs[0] == ".movies_imdb":
                if info["command"] == "PRIVMSG" and msgs[1] == "250movies":
                    methods["send"](info["address"], self.top_movies(movie2))
                if info["command"] == "PRIVMSG" and msgs[1] == "searchmovie":
                    if len(msgs) > 4:
                        methods["send"](info["address"], "too many messages")
                    elif len(msgs) == 4:
                        movie_to_search = msgs[2]
                        what_to_search = int(msgs[3])

                        if what_to_search == "cast":
                            methods["send"](info["address"], self.get_movie_cast(movie_to_search))
                        elif what_to_search == "directors":
                            methods["send"](
                                info["address"], self.get_movie_directors(movie_to_search)
                            )
                        elif what_to_search == "producers":
                            methods["send"](
                                info["address"], self.get_movie_producers(movie_to_search)
                            )
                        elif what_to_search == "rating":
                            methods["send"](info["address"], self.get_movie_rating(movie_to_search))
                        elif what_to_search == "year":
                            methods["send"](info["address"], self.get_movie_year(movie_to_search))
                        elif what_to_search == "genre":
                            methods["send"](info["address"], self.get_movie_genre(movie_to_search))
                        elif what_to_search == "runtime":
                            methods["send"](
                                info["address"], self.get_movie_runtime(movie_to_search)
                            )
                        elif what_to_search == "countries":
                            methods["send"](
                                info["address"], self.get_movie_countries(movie_to_search)
                            )

        except Exception as e:
            print("imdb plugin error: ", e)



"""
[joins.py]
Joins Plugin

[Author]
Gico Carlo Evangelista, https://gicocarlo.me/

[About]
Greets everyone who joins the channel
"""


class Plugin:
    def __init__(self):
        pass

    def run(self, incoming, methods, info, bot_info):
        try:
            # Parse the user ID from info['prefix']
            raw_user = info["prefix"]
            user_index = raw_user.find("!")
            user = raw_user[0:user_index]

            # If someone joins and it is not the bot, greet user
            bot_name = bot_info["name"]
            if info["command"] == "JOIN" and user != bot_name:

                # Greets joined user
                channel = info["args"][0]
                greet = f"Hello {user} welcome to {channel} !"
                methods["send"](info["address"], f"{greet}")

        except Exception as e:
            print("woops plugin error ", e)



NAME = "news.py"
ORIGINAL_AUTHORS = ["Angelo Giacco"]

ABOUT = """
Gets the top 10 headlines around the world from bbc world news
"""

COMMANDS = """
>>> .news
returns string of ten news headlines with newlines between
"""

WEBSITE = ""



NAME = "weather.py"
ORIGINAL_AUTHORS = ["Gabriele Ron"]

ABOUT = """
A plugin to get the weather of a location
"""

COMMANDS = """
>>> .weather <city> <country code>
returns the weather
"""

WEBSITE = ""



NAME = "diary.py"
ORIGINAL_AUTHORS = ["Angelo Giacco"]

ABOUT = """
Keeps a record of your diary
"""

COMMANDS = """
>>>.diary record <diary entry>
Adds to that day's diary entry

>>>.diary show <date>
Will show the diary entry of specified date (can also accept "today", "yesterday")

>>>.diary delete
Will delete today's entry

>>>.diary help
Gives help about how to use the plugin
"""

WEBSITE = ""



# development of cointoss plugin for honeybot
"""
[Toss.py]
A plugin to toss a coin

[Author]
Shriram Bhat

[About]
Responds to .cointoss, Returns head or tail at random

[commands]
>>> .cointoss
returns head/ tail
"""

import random


class Plugin:
    def __init__(self):
        pass

    def run(self, incoming, methods, info, bot_info):
        try:
            a = random.randint(0, 9)
            b = "It's Heads!"
            if a < 5:
                b = "It's Tails!"

            if info["command"] == "PRIVMSG" and info["args"][1] == ".cointoss":
                methods["send"](info["address"], b)
        except Exception as e:
            print("woops plug", e)



NAME = "converter.py"
ORIGINAL_AUTHORS = ["Angelo Giacco"]

ABOUT = """
Converts currencies
"""

COMMANDS = """
>>> .convert <<base currency code>> <<target currency code>> <<amount>>
returns the conversion: amount argument is optional with a default of 1
.convert help
shows a list of currencies supported
"""

WEBSITE = ""



NAME = "dadjoke.py"
ORIGINAL_AUTHORS = ["moffetma Oregon State University CS"]

ABOUT = """
Sends a random "dad joke" from icanhazdadjoke.com
"""

COMMANDS = """
>>> .dadjoke
Have you heard of the band 1023MB? They haven't got a gig yet.
"""

WEBSITE = ""



NAME = "translate.py"
ORIGINAL_AUTHORS = ["Ahmed Deeb"]

ABOUT = """
A Google Translate plugin
"""

COMMANDS = """
>>> . Google Translator object.

Examples
--------
translator = Plugin()

results1 = translator.translate("Einen schönen Tag allerseits")

# try 2 at a time
results2 = translator.translate(["Einen schönen Tag allerseits",
"Ich nehme an"])

# try detect
results3 = translator.detect("Einen schönen Tag allerseits")

# try to detect 2 at a time
results4 = translator.detect(["Einen schönen Tag allerseits",
"Ich nehme an"])
"""

WEBSITE = ""



"""
[story.py]
Story Plugin

[Author]
Tuan Thai

[About]
Sends a random story
Sourced stories from
http://www.read.gov/aesop/001.html

[Commands]
>>> .story
returns random story
"""

import random

import requests
from bs4 import BeautifulSoup


class Plugin:
    def __init__(self):
        pass

    def story(self):
        number = random.randint(2, 147)
        if number < 10:
            story = "00" + str(number)
        elif number < 100:
            story = "0" + str(number)
        else:
            story = str(number)

        story_url = requests.get("http://www.read.gov/aesop/" + story + ".html")
        soup = BeautifulSoup(story_url.content, "html.parser")
        texts = soup.find("div", {"id": "page"})
        for paragraph in texts.findAll("p"):
            return str(paragraph.text)

    def run(self, incoming, methods, info, bot_info):
        try:
            if info["command"] == "PRIVMSG" and info["args"][1] == ".story":
                methods["send"](info["address"], Plugin.story(self))
        except Exception as e:
            print("woops plugin error: ", e)



"""
[weather.py]
Weather Plugin

[Author]
Gabriele Ron

[About]
A plugin to get the weather of a location

[Commands]
>>> .weather <city> <country code>
returns the weather
"""

import requests


class Plugin:
    def __init__(self):
        pass

    def run(self, incoming, methods, info, bot_info):
        try:
            msgs = info["args"][1:][0].split()

            if info["command"] == "PRIVMSG" and msgs[0] == ".weather":
                api_url = (
                    "https://api.openweathermap.org/data/2.5/weather?"
                    f"q={msgs[1]},{msgs[2]}"
                    "&APPID=8801bc666d30a8cc9294feaf60c01117&units=metric"
                )
                response = requests.get(api_url)
                response_json = response.json()

                weather = (
                    f"The weather for {response_json['name']} is "
                    f"{response_json['weather'][0]['description']} at "
                    f"{int(response_json['main']['temp'])} degrees celsius"
                )
                methods["send"](info["address"], weather)
        except Exception as e:
            print("woops plugin error ", e)



"""
[comic.py]
Image search plugin

[Author]
Miguel Boekhold

[About]
Returns a random comic from xkcd

[Commands]
>>> .comic
returns a url of a random comic
"""

from random import randint

import requests


class Plugin:
    def __init__(self):
        pass

    def __comic():
        r = requests.get("http://xkcd.com/info.0.json")
        data = r.json()

        r = requests.get("http://xkcd.com/%d/info.0.json" % randint(1, data["num"]))
        data = r.json()
        image_url = data["img"]
        return image_url

    def run(self, incoming, methods, info, bot_info):
        try:
            if info["command"] == "PRIVMSG" and info["args"][1] == ".comic":
                methods["send"](info["address"], Plugin.__comic())
        except Exception as e:
            print("woops plugin error: ", e)



"""
[birthparadox.py]
Bithday Paradox Plugin

[Author]
Paulo Ferraz

[About]
show the probability that two or more people
have the same birthday in the given group.

[Commands]
>>> .birth <<number>>
returns the birthday matches and probability in % of a group.
"""
import datetime
import random


class Plugin:
    """
    Show the high probability that two or more people
    have the same birthday, even in a small group!
    """

    def __init__(self):
        pass

    """
    AUXILIARY FUNCTIONS
    """

    def calc_probability(self, num):
        """
        Calculate the match probability in %
        """
        soma = 366
        ano = 366
        for i in range(num - 1):
            soma *= ano - (i + 1)
        perc = round((1 - (soma / 366**num)) * 100, 2)
        return perc

    def create_birthdays(self, num_people):
        """
        Simulate some birthdays in a group
        """
        start_date = datetime.date(1970, 1, 1)
        birthday_group = []
        while True:
            birthday_group.clear()
            for i in range(num_people):
                birth = start_date + datetime.timedelta(random.randint(0, 366))
                birthday_group.append(birth.strftime("%B, %d"))
            if len(birthday_group) != len(set(birthday_group)):
                break
        return birthday_group

    def find_match(self, birthday_group):
        """
        Try to to find a match in a group
        """
        matches = []
        dates = ()
        for i, first in enumerate(birthday_group):
            for j, second in enumerate(birthday_group[i + 1 :]):
                if first == second:
                    matches.append(first)
        dates = set(matches)
        return dates

    def help(self, methods, info):
        """
        Presentation and guidance function
        """
        title = "THE BIRTHDAY PARADOX"
        statement = (
            "It's the high probability that two or more people have the same birthday, even in a"
            " small group!"
        )
        use = (
            "Use: .birth <<number>>, where num is the size of the group of people to calculate"
            " probability."
        )
        methods["send"](info["address"], title)
        methods["send"](info["address"], statement)
        methods["send"](info["address"], use)

    def run(self, incoming, methods, info, bot_info):
        """
        Main function
        """
        try:
            params = info["args"][1].split()
            if len(params) <= 1:
                if info["command"] == "PRIVMSG" and params[0] == ".birth":
                    Plugin.help(self, methods, info)
            if info["command"] == "PRIVMSG" and params[1]:
                if params[1].isdecimal() and int(params[1]) >= 5:
                    num = int(params[1])
                    perc = Plugin.calc_probability(self, num)
                    group = Plugin.create_birthdays(self, num)
                    dates = "; ".join(Plugin.find_match(self, group))
                    methods["send"](
                        info["address"],
                        "In this simulation, we found these [{}] matches, but on average, ".format(
                            dates
                        ),
                    )
                    methods["send"](
                        info["address"],
                        "this group has {} % of probability of having one or more people "
                        "with the same birthday!".format(perc),
                    )
                else:
                    methods["send"](info["address"], "Please, inform integer value >= 5 only!")
        except Exception as e:
            print("woops!! birthparadox plugin error ", e)



"""
[test.py]
Test Plugin

[Author]
Abdur-Rahmaan Janhangeer, pythonmembers.club

[About]
tests run
"""


class Plugin:
    def __init__(self):
        pass

    def run(self, incoming, methods, info, bot_info):
        # if '!~' in info['prefix']:
        # print(info)
        try:
            if info["command"] == "PRIVMSG" and info["args"][1] == ".test":
                methods["send"](info["address"], "test ok")
        except Exception as e:
            print("woops plug", e)



import configparser
import os
import time
import traceback
import unittest

from api.main import Bot_core as Bot

"""
':appinv!c5e342c5@gateway/web/cgi-irc/kiwiirc.com/ip.200.200.22.200 PRIVMSG ##bottestingmu :ef'
{
'prefix': 'appinv!c5e342c5@gateway/web/cgi-irc/kiwiirc.com/ip.200.200.22.200',
'command': 'PRIVMSG',
'address': '##bottestingmu',
'args': ['##bottestingmu', 'ef']
}
"""

# Setup info
os.chdir("src/honeybot/")
root = os.getcwd()
settings = os.path.join(root, "settings")
config = configparser.ConfigParser()
config.read("settings/CONNECT.conf")
info = {"settings_path": settings, "cwd": root}
# incoming
incoming = (
    ":appinv!c5e342c5@gateway/web/cgi-irc/kiwiirc.com/ip.200.200.22.200 PRIVMSG ##bottestingmu :ef"
)
bot = Bot(info)
sec = 5


class HoneybotTests(unittest.TestCase):
    """
    basic info
    """

    def test_name(self):
        self.assertEqual(bot.name, config["INFO"]["name"])

    def test_server_url(self):
        self.assertEqual(bot.server_url, config["INFO"]["server_url"])

    def test_port(self):
        self.assertEqual(bot.port, int(config["INFO"]["port"]))

    """
    info function
    """

    def test_info_prefix(self):
        self.assertEqual(
            bot.message_info(incoming)["prefix"],
            "appinv!c5e342c5@gateway/web/cgi-irc/kiwiirc.com/ip.200.200.22.200",
        )

    def test_info_command(self):
        self.assertEqual(bot.message_info(incoming)["command"], "PRIVMSG")

    def test_info_address(self):
        self.assertEqual(bot.message_info(incoming)["address"], "##bottestingmu")

    def test_info_args(self):
        self.assertEqual(bot.message_info(incoming)["args"], ["##bottestingmu", "ef"])


"""
commands
"""


def test_connect_command():
    unraised = True
    try:
        bot.connect()
        time.sleep(sec)
    except Exception as e:
        unraised = False
        print(repr(e))
        traceback.print_exc()
        assert unraised, "Connection Error!!!"
    finally:
        print("Testing IRC Connection...")


def test_join_command():
    unraised = True
    try:
        bot.join("#abc")
        time.sleep(sec)
    except Exception as e:
        unraised = False
        print(repr(e))
        traceback.print_exc()
        assert unraised, "Join Channel Error!!!"
    finally:
        print("Testing Joining Channel...")


def test_set_nick_command():
    unraised = True
    try:
        bot.greet()
        time.sleep(sec)
    except Exception as e:
        unraised = False
        print(repr(e))
        traceback.print_exc()
        assert unraised, "Nickname Setup Error!!!"
    finally:
        print("Testing Nickname Setup...")


def test_identify_command():
    unraised = True
    try:
        bot.identify()
        time.sleep(sec)
    except Exception as e:
        unraised = False
        print(repr(e))
        traceback.print_exc()
        assert unraised, "Identify Error!!!"
    finally:
        print("Testing Identifying Nickname...")


def test_specific_send_command():
    unraised = True
    try:
        bot.send_target("#abc", "greetings")
        time.sleep(sec)
    except Exception as e:
        unraised = False
        print(repr(e))
        traceback.print_exc()
        assert unraised, "Greetings Error!!!"
    finally:
        print("Testing Greetz...")


def test_pong_return():
    unraised = True
    try:
        bot.stay_alive("abc")
        time.sleep(sec)
    except Exception as e:
        unraised = False
        print(repr(e))
        traceback.print_exc()
        assert unraised, "Ping Error!!!"
    finally:
        print("Testing Ping... Pong...")


"""
Main Command Test
"""


def commands():
    test_connect_command()
    test_set_nick_command()
    test_join_command()
    test_identify_command()
    test_specific_send_command()
    test_pong_return()


if __name__ == "__main__":
    commands()
    unittest.main()



# player class


class Player:
    def __init__(self, name):
        # name will be the nickname in the IRC server
        # pot is the amount of money the player starts with
        # portfolio is an array with the indexes of the user's
        # properties from the board spaces dict
        self.name = name
        self.pot = 1500
        self.portfolio = []
        self.position = 0
        self.imprisoned = False
        self.prison_time = 0
        self.get_outta_jail_card = False

    def updatePosition(self, amount):
        passed_go = False
        self.position += amount
        if self.position > 39:
            passed_go = True
            self.position -= 40
            self.increasePot(200)
            if self.position > 39:
                self.position -= 40
        return passed_go

    def getName(self):
        return self.name

    def getPosition(self):
        return self.position

    def getPot(self):
        return str(self.pot)

    def increasePot(self, amount):
        self.pot += amount

    def reducePot(self, amount):
        if self.pot > amount:
            self.pot -= amount
            return True  # they are still alive
        else:
            self.pot = 0
            return False  # no longer alive

    def setPosition(self, index):
        # ensure index is between 0 and 39
        self.position = index

    def getPortfolio(self):
        return self.portfolio

    def checkWon():
        pass



NAME = "conv_sniff.py"
ORIGINAL_AUTHORS = ["Abdur-Rahmaan Janhangeer, pythonmembers.club"]

ABOUT = """
senses conversation topic with sensitivity set by user, also supports
word-specific sensing
"""

COMMANDS = """

"""

WEBSITE = ""



"""
[horoscope.py]
Horoscope Plugin

[Author]
Angelo Giacco

[About]
Returns your daily horoscope based on your star sign
you can also enter your month and date of birth to automatically find your starsign
enter month and date as number

[Commands]
>>> .horoscope <<starsign>>
returns horoscope for starsign

.horoscope <<month>> <<day>>
returns horoscope for starsign of day entered
"""
try:
    import requests
    from bs4 import BeautifulSoup
except ImportError:
    print("Required modules not found")


class Plugin:
    def __init__(self):
        pass

    def is_date_valid(month, day):
        day_count_for_month = [0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31]
        return 1 <= month <= 12 and 1 <= day <= day_count_for_month[month]

    def horoscope(starsign):
        signs = dict()
        signs["aries"] = "1"
        signs["taurus"] = "2"
        signs["gemini"] = "3"
        signs["cancer"] = "4"
        signs["leo"] = "5"
        signs["virgo"] = "6"
        signs["libra"] = "7"
        signs["scorpio"] = "8"
        signs["sagittarius"] = "9"
        signs["capricorn"] = "10"
        signs["aquarius"] = "11"
        signs["pisces"] = "12"

        if starsign not in signs:
            message = "That is not a valid star sign, check for typos"
        else:
            basic_url = (
                "https://www.horoscope.com/us/horoscopes/general"
                "/horoscope-general-daily-today.aspx?sign="
            )
            url = basic_url + signs[starsign]  # creates the url
            content = requests.get(url)  # gets the horoscope website
            doc = BeautifulSoup(content.text, "html.parser")  # parses website
            message = doc.find_all("p")[0].text  # the first p element of the website
            # is the horoscope and we only want its text
        return message

    def run(self, incoming, methods, info, bot_info):
        try:
            msgs = info["args"][1:][0].split()

            if info["command"] == "PRIVMSG" and msgs[0] == ".horoscope":
                # The next message should be the star sign
                if (
                    len(msgs) > 3
                ):  # the message should only contain .horoscope and the user's star sign
                    methods["send"](info["address"], "too many messages")
                elif len(msgs) == 3:
                    month = msgs[1]
                    day = int(msgs[2])

                    def is_date_valid(month, day):
                        day_count_for_month = [
                            0,
                            31,
                            28,
                            31,
                            30,
                            31,
                            30,
                            31,
                            31,
                            30,
                            31,
                            30,
                            31,
                        ]
                        return 1 <= month <= 12 and 1 <= day <= day_count_for_month[month]

                    # message = "month: "+month+" and day:"+day
                    # methods['send'](info['address'], message)

                    if month.lower() == "december" or month == "12":
                        if is_date_valid(12, day):
                            astro_sign = "sagittarius" if (day < 22) else "capricorn"
                            methods["send"](info["address"], Plugin.horoscope(astro_sign))
                        else:
                            methods["send"](info["address"], "invalid day")
                    elif month.lower() == "january" or month == "1":
                        if is_date_valid(1, day):
                            astro_sign = "capricorn" if (day < 20) else "aquarius"
                            methods["send"](info["address"], Plugin.horoscope(astro_sign))
                        else:
                            methods["send"](info["address"], "invalid day")
                    elif month.lower() == "february" or month == "2":
                        if is_date_valid(2, day):
                            astro_sign = "aquarius" if (day < 19) else "pisces"
                            methods["send"](info["address"], Plugin.horoscope(astro_sign))
                        else:
                            methods["send"](info["address"], "invalid day")
                    elif month.lower() == "march" or month == "3":
                        if is_date_valid(3, day):
                            astro_sign = "pisces" if (day < 21) else "aries"
                            methods["send"](info["address"], Plugin.horoscope(astro_sign))
                        else:
                            methods["send"](info["address"], "invalid day")
                    elif month.lower() == "april" or month == "4":
                        if is_date_valid(4, day):
                            astro_sign = "aries" if (day < 20) else "taurus"
                            methods["send"](info["address"], Plugin.horoscope(astro_sign))
                        else:
                            methods["send"](info["address"], "invalid day")
                    elif month.lower() == "may" or month == "5":
                        if is_date_valid(5, day):
                            astro_sign = "taurus" if (day < 21) else "gemini"
                            methods["send"](info["address"], Plugin.horoscope(astro_sign))
                        else:
                            methods["send"](info["address"], "invalid day")
                    elif month.lower() == "june" or month == "6":
                        if is_date_valid(6, day):
                            astro_sign = "gemini" if (day < 21) else "cancer"
                            methods["send"](info["address"], Plugin.horoscope(astro_sign))
                        else:
                            methods["send"](info["address"], "invalid day")
                    elif month.lower() == "july" or month == "7":
                        if is_date_valid(7, day):
                            astro_sign = "cancer" if (day < 23) else "leo"
                            methods["send"](info["address"], Plugin.horoscope(astro_sign))
                        else:
                            methods["send"](info["address"], "invalid day")
                    elif month.lower() == "august" or month == "8":
                        if is_date_valid(8, day):
                            astro_sign = "leo" if (day < 23) else "virgo"
                            methods["send"](info["address"], Plugin.horoscope(astro_sign))
                        else:
                            methods["send"](info["address"], "invalid day")
                    elif month.lower() == "september" or month == "9":
                        if is_date_valid(9, day):
                            astro_sign = "virgo" if (day < 23) else "libra"
                            methods["send"](info["address"], Plugin.horoscope(astro_sign))
                        else:
                            methods["send"](info["address"], "invalid day")
                    elif month.lower() == "october" or month == "10":
                        if is_date_valid(10, day):
                            astro_sign = "libra" if (day < 23) else "scorpio"
                            methods["send"](info["address"], Plugin.horoscope(astro_sign))
                        else:
                            methods["send"](info["address"], "invalid day")
                    elif month.lower() == "november" or month == "11":
                        if is_date_valid(11, day):
                            astro_sign = "scorpio" if (day < 22) else "sagittarius"
                            methods["send"](info["address"], Plugin.horoscope(astro_sign))
                        else:
                            methods["send"](info["address"], "invalid day")
                    else:
                        methods["send"](info["address"], ".horoscope requires a valid month")
                elif len(msgs) == 2:
                    starsign = msgs[1]
                    methods["send"](info["address"], Plugin.horoscope(starsign))
                else:
                    methods["send"](
                        info["address"],
                        ".horoscope requires a star sign or day and month",
                    )

        except Exception as e:
            print("woops horoscope plugin error: ", e)



NAME = "dictionary.py"
ORIGINAL_AUTHORS = ["Milad H,"]

ABOUT = """
Searches the python documentatioin(pydocs) for the related keyword(s)
and returns some of the relevant links
"""

COMMANDS = """
>>> .pydocs <keyword>
returns links in python documentation related to the keyword
"""

WEBSITE = ""



NAME = "book_of_day.py"

ORIGINAL_AUTHORS = ["Schoberg", "UMD"]

ABOUT = """book_of_day
Using bookoftheday.org, get the title and link of the book of the day
"""

COMMANDS = """
>>> .bookoftheday
https://bookoftheday.org/fleishman-is-in-trouble-taffy-brodesser-akner/
"""



"""
[googleSearch.py]
Google Search Plugin

[Author]
Justin Walker

[About]
Returns the first three links from a google search.

[Commands]
>>> .google <<search term>>
returns search links
"""

try:
    from googlesearch import search
except ImportError:
    print("No module named 'google' found")


class Plugin:
    def __init__(self):
        pass

    def __google(search_term):
        # start is what link to start with, stop is how many links to get
        # only_standard limits it to normal links instead of ads and extra
        # links.
        return search(search_term, start=1, stop=3, only_standard=True)

    def run(self, incoming, methods, info, bot_info):
        try:
            msgs = info["args"][1:][0].split()

            if info["command"] == "PRIVMSG" and msgs[0] == ".google":
                # All further messages, if there are any are added to search term.
                term = ""
                if len(msgs) > 1:
                    for msg in msgs[1:]:
                        term += msg
                    for link in Plugin.__google(term):
                        methods["send"](info["address"], link)
                else:
                    methods["send"](info["address"], "Input error. '.google search_term'.")

        except Exception as e:
            print("woops plugin error: ", e)



"""
[pynews.py]
Python news checking plugin

[Author]
Sam Deans

[Github]
@sdeans0

[About]
Finds the top five news items on python.org and prints them to the IRC

[Commands]
>>> .pynews
returns the top 5 python.org news articles
"""

import requests
from bs4 import BeautifulSoup


class Plugin:
    def __init__(self):
        pass

    def run(self, incoming, methods, info, bot_info):
        try:
            if info["command"] == "PRIVMSG" and info["args"][1] == ".pynews":
                # Get python website
                response = requests.get("https://www.python.org")
                html = BeautifulSoup(response.text)
                # The news section is the first <div class='shrubbery'> in the page
                # So get all the links in that section
                news_links = (
                    html.find("div", class_="shrubbery").find("ul", class_="menu").find_all("a")
                )
                # Iterate through the links, send each one to the chat with its
                # associated title (a.contents[0])
                for a in news_links:
                    methods["send"](info["address"], ": ".join([a.contents[0], a["href"]]))
        except Exception as e:
            print("woops plugin", __file__, e)



NAME = "username.py"
ORIGINAL_AUTHORS = ["Abdur-Rahmaan Janhangeer, pythonmembers.club"]

ABOUT = """
generates random usernames, used for inspiration
"""

COMMANDS = """
>>> .uname
returns generated username
"""

WEBSITE = ""



"""
[wikipedia.py]
Wikipedia Plugin

[Author]
Gabriele Ron

[About]
sends a wikipedia article on request based off of a search or random query
requires wikipedia library. Can use pip -> "pip install wikipedia"

[Website]
https://Macr0Nerd.github.io

[Commands]
>>> .wiki <command> <topic>
returns a wikipedia article
"""

import wikipedia


class Plugin:
    def __init__(self):
        pass

    def run(self, incoming, methods, info, bot_info):
        # if '!~' in info['prefix']:
        # print(info)
        try:
            msgs = info["args"][1:][0].split()

            if info["command"] == "PRIVMSG" and msgs[0] == ".wiki":
                if msgs[1] == "random":
                    print("You want the random article!")

                    article = wikipedia.page(wikipedia.random(1)).summary
                    methods["send"](info["address"], article)
                elif msgs[1] == "search":
                    print(f"You wanted {msgs[2]}")

                    article = wikipedia.page(msgs[2]).summary
                    methods["send"](info["address"], article)
        except Exception as e:
            print("woops plug", e)



NAME = "hangman.py"
ORIGINAL_AUTHORS = ["Justin Walker"]

ABOUT = """
Plays game of hangman
"""

COMMANDS = """
>>> .hangman start
Starts hangman game and returns game screen.

>>> .hangman guess <<word or letter>>
Submits word or letter guess and returns game screen.
"""

WEBSITE = ""



"""
[quote.py]
Quotes Plugin

[Author]
German Corpaz

[About]
Sends a random quote on request

[Commands]
>>> .quote
returns a random quote
"""
import random


class Plugin:
    def __init__(self):
        pass

    def quote(self):
        quotes = [
            "You know you’re in love when you can’t fall asleep because "
            + "reality is finally better than your dreams.",
            "Life is what happens when you’re busy making other plans.",
            "Get busy living or get busy dying.",
            "Twenty years from now you will be more disappointed by "
            + "the things that you didn’t do than by the ones you did do.",
            "The best and most beautiful things in the world cannot be seen "
            + "or even touched - they must be felt with the heart.",
            "Great minds discuss ideas; average minds discuss events; "
            + "small minds discuss people.",
            "Keep love in your heart. A life without it is like a "
            + "sunless garden when the flowers are dead.",
            "A successful man is one who can lay a firm foundation "
            + "with the bricks others have thrown at him.",
            "It is during our darkest moments that we must focus to see the light.",
            "Those who dare to fail miserably can achieve greatly.",
            "Try to be a rainbow in someone's cloud.",
            "I can’t give you a sure-fire formula for success, but I can give you "
            + "a formula for failure: try to please everybody all the time.",
            "Find a place inside where there's joy, and the joy will burn out the pain.",
            "He that falls in love with himself will have no rivals.",
            "Nothing is impossible, the word itself says 'I'm possible'!",
            "It is hard to fail, but it is worse never to have tried to succeed.",
            "Don't judge each day by the harvest you reap " + "but by the seeds that you plant.",
            "Do not dwell in the past, do not dream of the future, "
            + "concentrate the mind on the present moment.",
            "I’m a success today because I had a friend who believed in me "
            + "and I didn’t have the heart to let him down.",
            "The only thing necessary for the triumph of evil " + "is for good men to do nothing.",
            "Love yourself first and everything else falls into line. "
            + "You really have to love yourself to get anything done in this world.",
            "One of the most beautiful qualities of true friendship "
            + "is to understand and to be understood.",
            "Let us always meet each other with smile, "
            + "for the smile is the beginning of love.",
            "Where there is love there is life.",
            "Challenges are what make life interesting and "
            + "overcoming them is what makes life meaningful.",
            "Love is composed of a single soul inhabiting two bodies.",
            "Love is a serious mental disease.",
            "Do not go where the path may lead, go instead "
            + "where there is no path and leave a trail.",
            "Our greatest fear should not be of failure… but of succeeding "
            + "at things in life that don’t really matter.",
            "Success is not final, failure is not fatal: "
            + "it is the courage to continue that counts.",
            "Remember that the happiest people are not those "
            + "getting more, but those giving more.",
            "Do all things with love.",
            "The only impossible journey is the one you never begin.",
            "Blessed are the hearts that can bend; they shall never be broken.",
            "It is our choices, that show what we truly are, " + "far more than our abilities.",
            "Love isn't something you find. Love is something that finds you.",
            "Only put off until tomorrow what you are willing to die having left undone.",
            "In three words I can sum up everything I've learned about life: it goes on.",
            "If you want to be happy, be.",
            "Change your thoughts and you change your world.",
            "Education is the most powerful weapon which " + "you can use to change the world.",
            "Today you are you! That is truer than true! "
            + "There is no one alive who is you-er than you!",
            "The future belongs to those who believe in the beauty of their dreams.",
        ]

        return f"{random.choice(quotes)}"

    def run(self, incoming, methods, info, bot_info):
        try:
            msgs = info["args"][1:]
            if info["command"] == "PRIVMSG" and msgs[0] == ".quote":
                methods["send"](info["address"], Plugin.quote(self))
        except Exception as e:
            print("error quote plugin", e)



"""
[russian_roulette.py]
Russian Roulette Plugin

[Author]
Angelo Giacco

[About]
In the original russian roulette, you have a one sixth chance of committing
suicide. In this version, you have a one sixth chance of being kicked from
the channel. The bot will utilise the IRC command:
KICK #channel nickname
where #channel is the channel name and nicknames is the nickname of the user

WARNING: BOT MUST BE CHANNEL OPERATOR!!!

[Commands]
>>> .russian_roulette
either returns a string saying you survived or kicks you off the channel
"""
import random


class Plugin:
    def __init__(self):
        pass

    def risk(self, incoming, methods, info):
        kill = True if random.random() < 0.2 else False  # should the user be kicked
        if kill:
            name = info["prefix"].split("!")[0]
            channel = info["address"]
            kill_command = "KICK " + channel + " " + name + " \r\n"
            methods["send_raw"](kill_command)
            # code to quit the channel
            return "Suicide is always a risk when playing russian roulette... RIP..."
        else:
            return "You survived..."

    def run(self, incoming, methods, info, bot_info):
        try:
            if info["command"] == "PRIVMSG" and info["args"][1] == ".russian_roulette":
                methods["send"](info["address"], Plugin.risk(self, incoming, methods, info))
        except Exception as e:
            print("woops russian roulette plugin error ", e)



NAME = "help.py"
ORIGINAL_AUTHORS = ["Eduardo Moraes de Mello, https://github.com/edumello"]

ABOUT = """
Show link with all the functionalities of the bot.
"""

COMMANDS = """
>>> .help
returns link of plugins informations
"""

WEBSITE = ""



import configparser
import importlib
import logging
import os
import pathlib
import socket
import subprocess
import sys
import time

import pkg_resources
import tomli

from honeybot.api import commands, memory
from honeybot.api import print as output
from honeybot.api.utils import get_requirements, prevent_none

plugins = []

# Start logger
logger = logging.getLogger("bot_core")

"""
BOT CONNECTION SETUP
"""


class BotCore:
    """
    The main class that represents the core functionality of the bot.

    Args:
        info (dict): A dictionary containing information about the bot.
        password (str, optional): The password for the bot. Defaults to "".

    Attributes:
        info (dict): A dictionary containing information about the bot.
        configs (dict): A dictionary containing the bot's configurations.
        settings_path (str): The path to the bot's settings file.
        root_path (str): The root path of the bot.
        server_url (str): The URL of the server the bot connects to.
        port (int): The port number the bot uses to connect to the server.
        name (str): The name of the bot.
        owners (list): A list of usernames who are the owners of the bot.
        password (str): The password for the bot.
        friends (list): A list of usernames who are friends of the bot.
        autojoin_channels (list): A list of channels the bot automatically joins.
        downloaded_plugins_to_load (list): A list of downloaded plugins to load.
        required_modules (list): A list of required modules for the bot.
        time (float): The current time.
        irc (socket.socket): The socket object for the bot's IRC connection.
        is_listen_on (int): A flag indicating whether the bot is listening for incoming messages.
        domain (str): The domain of the server the bot connects to.
        sp_command (str): The special command for the bot.
        plugins (list): A list of loaded plugins.
        core_plugins (list): A list of core plugins.

    Methods:
        message_info(s): Parses a message string and returns a dictionary containing information about the message.
        bot_info(): Returns a dictionary containing information about the bot.
        methods(): Returns a dictionary of methods that can be used by plugins.
        send(msg): Sends a raw message to the server.
        send_target(target, msg): Sends a message to a specific target (channel or user).
        join(channel): Joins a channel.
        is_valid_plug_name(name): Checks if a plugin name is valid.
        print_running_infos(): Prints the running information of the bot.
        load_plugins_from_folder(category_folder, from_conf, from_dir): Loads plugins from a folder.
        load_plugins(): Loads the plugins specified in the plugins list.
        run_plugins(incoming): Runs the loaded plugins on an incoming message.
        core_commands_parse(incoming): Parses the core commands in an incoming message.
        connect(): Connects to the server.
        identify(): Sends the identification command to the server.
        greet(): Sends the initial commands to the server after connecting.
        pull(): Listens for incoming messages from the server.
        quit(): Sends the quit command to the server.
        stay_alive(incoming): Handles the stay alive functionality of the bot.
        registered_run(): Runs the bot as a registered bot.
        unregistered_run(): Runs the bot as an unregistered bot.
    """

    def __init__(self, info, password=""):
        self.info = info
        with open(info["toml_path"], "rb") as f:
            self.configs = tomli.load(f)
        self.settings_path = self.info["settings_path"]
        self.root_path = self.info["cwd"]
        self.server_url = self.configs["INFO"]["server_url"]
        self.port = int(self.configs["INFO"]["port"])
        self.name = self.configs["INFO"]["name"]
        self.owners = self.configs["USERNAMES"]["owners"]
        self.password = password
        self.friends = self.configs["USERNAMES"]["friends"]
        self.autojoin_channels = self.configs["INFO"]["autojoin_channels"]
        self.downloaded_plugins_to_load = self.configs["PLUGINS"]["downloaded"]
        self.required_modules = get_requirements()
        self.time = time.time()

        self.irc = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.is_listen_on = 1
        dom = self.server_url.split(".")
        self.domain = ".".join(dom[-2:])
        self.sp_command = "hbot"
        self.plugins = []
        self.core_plugins = []

    """
    MESSAGE VALIDATION
    """

    def message_info(self, s):
        """
        Parses a message string and returns a dictionary containing information about the message.

        Args:
            s (str): The message string to parse.

        Returns:
            dict: A dictionary containing the following keys:
                - 'prefix': The prefix of the message (if present).
                - 'command': The command of the message.
                - 'args': A list of arguments of the message.
                - 'address': The address of the message (either a channel or a user).
                - 'user': The user who sent the message.

        Raises:
            None.

        """
        try:
            prefix = ""
            trailing = []
            address = ""
            if not s:
                pass
            if s[0] == ":":
                prefix, s = s[1:].split(" ", 1)
            if s.find(" :") != -1:
                s, trailing = s.split(" :", 1)
                args = s.split()
                args.append(trailing)
            else:
                args = s.split()
            command = args.pop(0)
            address = args[0] if "#" in args[0] else prefix.split("!~")[0]
            user = prefix.split("!")[0]
            return {
                "prefix": prevent_none(prefix),
                "command": prevent_none(command),
                "args": ["" if e is None else e for e in args],
                "address": prevent_none(address),
                "user": prevent_none(user),
            }
        except Exception as e:
            pass

    def bot_info(self):
        return {
            "name": self.name,
            "special_command": self.sp_command,
            "required_modules": self.required_modules,
            "owners": self.owners,
            "time": self.time,
            "friends": self.friends,
        }

    def methods(self):
        return {
            "send_raw": self.send,
            "send": self.send_target,
            "join": self.join,
            "mem_add": memory.add_value,
            "mem_rem": memory.remove_value,
            "mem_fetch": memory.fetch_value,
        }

    """
    MESSAGE UTIL
    """

    def send(self, msg):
        self.irc.send(bytes(msg, "UTF-8"))

    def send_target(self, target, msg):
        self.send(commands.specific_send(target, msg))

    def join(self, channel):
        self.send(commands.join_channel(channel))

    """
    PLUGIN UTILS
    """

    def is_valid_plug_name(self, name):
        """
        Checks if the given plug name is valid.

        Args:
            name (str): The name of the plug.

        Returns:
            bool: True if the name is valid, False otherwise.
        """
        if name.startswith("__") or name == "":
            return False

        return True

    """
    BOT UTIL
    """

    def print_running_infos(self):
        print(output.status("i") + " Run infos:")
        for key in self.info:
            print(output.tab() + " " + key, self.info[key])
        print(output.line())

    def load_plugins_from_folder(self, category_folder, from_conf=None, from_dir=None):
        """
        Load plugins from a specified folder.

        Args:
            category_folder (str): The category folder from which to load the plugins.
            from_conf (bool, optional): If True, load plugins specified in the configuration file. Defaults to None.
            from_dir (bool, optional): If True, load plugins from the core directory. Defaults to None.
        """
        if from_dir is True:
            dir_path = os.path.join(self.info["plugins_path"], "core")
            to_load = [f for f in os.listdir(dir_path) if self.is_valid_plug_name(f)]
        elif from_conf is True:
            to_load = self.configs["PLUGINS"]["downloaded"]

        print(output.status("i") + " Loading from", category_folder)
        for folder in to_load:
            print(output.tab(), "loading plugin:", folder)
            try:
                sys.path.append(self.root_path)
                module = importlib.import_module(f"plugins.{category_folder}.{folder}.main")
                obj = module
                self.plugins.append(obj)
            except ModuleNotFoundError as e:
                logger.warning(f"{folder}: module import error, skipped' {e}")

            try:
                req_path = os.path.join(
                    self.info["cwd"],
                    "plugins",
                    category_folder,
                    folder,
                    "requirements.txt",
                )
                if os.path.exists(req_path):
                    with pathlib.Path(req_path).open() as requirements_txt:
                        install_requires = [
                            str(requirement)
                            for requirement in pkg_resources.parse_requirements(requirements_txt)
                        ]
                        print("installing", install_requires)
                        subprocess.check_call(
                            [sys.executable, "-m", "pip", "install", *install_requires]
                        )
            except Exception as e:
                # logger.debug(e)
                pass

    def load_plugins(self):
        """
        Load plugins from the 'downloaded' and 'core' folders.

        This method loads plugins from two folders: 'downloaded' and 'core'. It first loads plugins from the 'downloaded'
        folder using the 'load_plugins_from_folder' method with the 'from_conf' parameter set to True. Then, it loads plugins
        from the 'core' folder using the same method with the 'from_dir' parameter set to True.

        After loading the plugins, it prints a status message indicating that the plugins have been loaded.

        Returns:
            None
        Examples:
            TODO
        """
        print(output.status("i") + " Loading plugins...")

        self.load_plugins_from_folder("downloaded", from_conf=True)
        self.load_plugins_from_folder("core", from_dir=True)

        print(output.status("x") + " Loaded plugins")
        print(output.line())

    def run_plugins(self, incoming):
        """
        Runs the plugins on the incoming message.

        Args:
            incoming (str): The incoming message.

        Returns:
            str: The modified incoming message after running the plugins.
        """
        for plugin in self.plugins:
            P = getattr(plugin, "Plugin")
            incoming = incoming
            methods = self.methods()
            info = self.message_info(incoming)
            bot_info = self.bot_info()
            hbot_plugin = P()
            hbot_plugin.run(incoming, methods, info, bot_info)

    """
    MESSAGE PARSING
    """

    def core_commands_parse(self, incoming):
        self.run_plugins(incoming)

    """
    BOT IRC FUNCTIONS
    """

    def connect(self):
        self.irc.connect((self.server_url, self.port))

    def identify(self):
        self.send(commands.identify(self.password))

    def greet(self):
        self.send(commands.set_nick(self.name))
        self.send(commands.present(self.name))
        for channel in self.autojoin_channels:
            self.send(commands.join_channel(channel))
        print(output.status("x"), "Joined channels:", ", ".join(self.autojoin_channels))

    def pull(self):
        """
        Listens to incoming messages and parses them using core_commands_parse method.
        """
        print(output.status("i"), "Listening to incoming messages")
        while self.is_listen_on:
            try:
                data = self.irc.recv(2048).decode("UTF-8", errors="replace")
                for line in data.split("\r\n"):
                    if line != "":
                        self.core_commands_parse(line)

            except KeyboardInterrupt:
                self.is_listen_on = False
                self.quit()
            except Exception as e:
                pass

    def quit(self):
        self.send(commands.quit())
        self.is_listen_on = False

    """
    ONGOING REQUIREMENT/S
    """

    def stay_alive(self, incoming):
        """
        Handles the stay alive functionality of the bot.

        Args:
            incoming (str): The incoming message.

        Returns:
            None
        """
        if not incoming:
            logger.critical("<must handle reconnection - incoming is not True>")
            sys.exit()
        parts = incoming.split(":")
        if parts[0].strip().lower() == "ping":
            logger.warning(f"ping detected from: {parts[1]}")
            self.send(commands.pong_return(self.domain))
            self.send(commands.pong_return(parts[1]))

    # all in one for registered bot
    def registered_run(self):
        self.connect()
        self.identify()
        self.greet()
        self.load_plugins()
        self.pull()

    def unregistered_run(self):
        self.print_running_infos()
        self.connect()
        self.greet()
        self.load_plugins()
        self.pull()



NAME = "password_generator.py"
ORIGINAL_AUTHORS = ["Nishant, JPMorgan Chase & Co."]

ABOUT = """
sends different type of passwords
"""

COMMANDS = """
>>> .passgen <<length>>
returns alphabetic password of specified length

>>> .passgensecure <<length>>
returns secure alphanumeric password of specified length

>>> .passgenalphanum <<length>>
returns alphanumeric password of specified length

>>> .passgenspecialchar <<length>>
returns universally accepted alphanumeric password
with special characters of specified length where length >= 4
"""

WEBSITE = ""



""" Change to your plugin file name below.
                    ↓↓↓↓  """
import time

from plugins import basketball as test_plugin

test_plugin = test_plugin.Plugin


class Tester:
    def __init__(self, test_type, command):

        # Same methods as HoneyBot, but these print to screen
        methods = {"send_raw": self.send, "send": self.send_target, "join": self.join}

        # Set MOCK info for testing
        info = {
            "address": "##TestChannel",
            "args": ["##TestChannel"],
            "bot_name": "honeybot",
            "bot_special_command": "hbot",
            "command": "QUIT",
            "prefix": "TestAccount!~TestIdentity@192.168.123.123",
        }
        bot_info = {"time": time.time() - 12561}
        # time.sleep(5)

        # Update info with MOCK data.
        if test_type == "message":
            incoming = (
                ":TestAccount!~TestIdentity@192.168.123.123                 PRIVMSG ##TestChannel"
                f" :{command}"
            )
            info["command"] = "PRIVMSG"
            info["args"] = [0, command]

        if test_type == "user_join":
            incoming = (
                ":TestAccount!~TestIdentity@192.168.123.123                 JOIN ##TestChannel"
            )
            info["command"] = "JOIN"

        if test_type == "user_quit":
            incoming = ":TestAccount!~TestIdentity@192.168.123.123 QUIT :"
            info["command"] = "QUIT"

        # Run plugin with above details
        test_plugin.run(self, incoming, methods, info, bot_info)

    def send(self, msg):
        print(msg)

    def join(self, channel):
        print(f"JOINING CHANNEL: {channel}")

    def send_target(self, target, msg):
        print(f"TO CHANNEL: {target}")
        print(f"MESSAGE: {msg}")


if __name__ == "__main__":
    """
    HOW TO USE:
        1. Add Plugin to plugins folder.
        2. Change plugin name at the top of this file.
        3. message
            3.1 Uses command as if a user was testing sending a message.
        4. join
            4.1 User joining a channel
        4. quit
            4.1 User quitting a channel
    """

    incoming_command_test = ".basketball eu all"

    Tester("message", incoming_command_test)
    # Tester('user_join', incoming_command_test)
    # Tester('user_quit', incoming_command_test)



"""
[book_of_day.py]
Book of the Day Plugin

[Author]
Schoberg, UMD

[About]
using bookoftheday.org, get the title and link of the book of the day

[Commands]
>>> .bookoftheday
https://bookoftheday.org/fleishman-is-in-trouble-taffy-brodesser-akner/
"""

import requests
from bs4 import BeautifulSoup


class Plugin:
    def __init__(self):
        pass

    def book(self):
        headers = {"Accept": "application/json"}
        r = requests.get("http://bookoftheday.org", headers=headers)
        soup = BeautifulSoup(r.text, "html.parser")
        d = soup.find_all("div", {"class": "chpcs_foo_content"})

        return str(d[0].a["href"])

    def run(self, incoming, methods, info, bot_info):
        try:
            if info["command"] == "PRIVMSG" and info["args"][1] == ".bookofday":
                methods["send"](info["address"], Plugin.book(self))
        except Exception as e:
            print("woops book_of_day plugin error", e)



"""
[monero.py]
Monero Price Checking Plugin

[Author]
A. Eigenbrot


[About]
Checks the current price for Monero via the cryptocompare.com API

[Commands]
>>> .xmr [currency]
returns current value of Monero in the currency specified (default is USD).
Currency must be a 3-letter capital string.
Examples: USD, EUR, BTC, GBP
"""

import json

from requests import Session


class Plugin:
    def __init__(self):
        pass

    def run(self, incoming, methods, info, bot_info):
        try:
            msgs = info["args"][1:][0].split()
            if info["command"] == "PRIVMSG" and msgs[0] == ".xmr":
                try:
                    currency = msgs[1].upper()
                except IndexError:
                    currency = "USD"

                api_url = "https://min-api.cryptocompare.com/data/price"
                params = {"fsym": "XMR", "tsyms": currency}

                session = Session()
                headers = {"Accepts": "application/json"}
                session.headers.update(headers)

                result = session.get(api_url, params=params)
                data = json.loads(result.text)
                methods["send"](info["address"], f"{data[currency]} {currency}")

        except Exception as e:
            print("woops! monero plugin error:", e)



""" card class """

# pylint: disable=E1601


class Card:
    """card class"""

    def __init__(self, card):
        """card initialization; Card('8s'), Card('QC')"""
        values = [
            0,
            1,
            2,
            3,
        ]  # 0 is for a low: 2 3 4,
        # 1 for a medium 4 5 6 7,
        # 2 for a high 8 9 10,
        # 3 for a suit J Q K A

        self.__figure = card[0]
        self.__color = card[1].upper()
        self.__card = self.__figure + self.__color

        if self.__figure == "A":
            self.__value = 14
            self.__21_value = 11

        elif self.__figure == "K":
            self.__value = 13
            self.__21_value = 10

        elif self.__figure == "Q":
            self.__value = 12
            self.__21_value = 10

        elif self.__figure == "J":
            self.__value = 11
            self.__21_value = 10

        elif self.__figure == "T":
            self.__value = 10
            self.__21_value = 10

        else:
            self.__value = int(self.__figure)
            self.__21_value = int(self.__figure)

    def show_card(self):
        """show card"""
        if self.__card == "0X":
            print("This card does not exist, check your index!")
            pass
        else:
            return self.__card

    def figure(self):
        """show figure"""

        return self.__figure

    def color(self):
        """show color"""

        return self.__color

    def value(self):
        """show value"""

        return self.__value

    def blackjack_value(self):
        """show value in the game of blackjack"""

        return self.__21_value



NAME = "date.py"
ORIGINAL_AUTHORS = ["Gico Carlo Evangelista, https://gicocarlo.me/"]

ABOUT = """
Will post the current date
"""

COMMANDS = """
>>> .date today
returns the current date
"""

WEBSITE = ""



"""
[password_generator.py]
Password Generator Plugin

[Author]
Nishant, JPMorgan Chase & Co.

[About]
sends different type of passwords

[Commands]
>>> .passgen <<length>>
returns alphabetic password of specified length

>>> .passgensecure <<length>>
returns secure alphanumeric password of specified length

>>> .passgenalphanum <<length>>
returns alphanumeric password of specified length

>>> .passgenspecialchar <<length>>
returns universally accepted alphanumeric password with
special characters of specified length where length >= 4
"""

import random
import secrets
import string


class Plugin:
    def __init__(self):
        pass

    def __passgen(password_length):
        letters = string.ascii_lowercase
        return "".join(random.choice(letters) for i in range(password_length))

    def __passgenalphanum(password_length):
        password_characters = string.ascii_letters + string.digits + string.punctuation
        return "".join(random.choice(password_characters) for i in range(password_length))

    def __passgensecure(password_length):
        password_characters = string.ascii_letters + string.digits + string.punctuation
        return "".join(secrets.choice(password_characters) for i in range(password_length))

    def __passgenspecialchar(password_length):
        randomSource = string.ascii_letters + string.digits + string.punctuation
        password = random.choice(string.ascii_lowercase)
        password += random.choice(string.ascii_uppercase)
        password += random.choice(string.digits)
        password += random.choice(string.punctuation)
        for i in range(password_length - 4):
            password += random.choice(randomSource)
        passwordList = list(password)
        random.SystemRandom().shuffle(passwordList)
        password = "".join(passwordList)
        return password

    def __securePasswordGenerator():
        password = ""
        person = ["I", "i", "He", "he", "She", "she", "It", "it"]
        verb = [
            ["am", "is"],
            ["run", "runs"],
            ["go", "goes"],
            ["drink", "drinks"],
            ["eat", "eats"],
            ["sleep", "sleeps"],
            ["like", "likes"],
        ]
        prep = ["to", "under", "along", "within", "until", "into"]
        article = ["the", "a"]
        thing = ["bridge", "car", "store", "shirt", "computer", "bed", "couch"]

        password += person[random.randint(0, len(person) - 1)]

        if password == person[0] or password == person[1]:
            password += verb[random.randint(0, len(verb) - 1)][0]
        else:
            password += verb[random.randint(0, len(verb) - 1)][1]

        password += prep[random.randint(0, len(prep) - 1)]
        password += article[random.randint(0, len(article) - 1)]
        password += thing[random.randint(0, len(thing) - 1)]

        char_converter = {"i": "1", "e": "3", "a": "@", " ": "-", "o": "0"}
        i = random.randint(0, 2)
        new = list(password)

        if i % 2 == 0:
            for i in range(len(password)):
                if new[i] in char_converter:
                    new[i] = char_converter[new[i]]
            new = "".join(new)
            return new

        else:
            num = round(len(password) / 2)
            for i in range(num + 1):
                if new[i] in char_converter:
                    new[i] = char_converter[new[i]]
            new = "".join(new)
            return new

    def run(self, incoming, methods, info, bot_info):
        try:
            # if '!~' in info['prefix']:
            # print(info)
            msgs = info["args"][1:][0].split()
            if info["command"] == "PRIVMSG":
                if len(msgs) > 1:
                    if msgs[0] == ".passgen":
                        length = msgs[1]
                        methods["send"](info["address"], Plugin.__passgen(int(length)))
                    if msgs[0] == ".passgensecure":
                        length = msgs[1]
                        methods["send"](info["address"], Plugin.__passgensecure(int(length)))
                    if msgs[0] == ".passgenalphanum":
                        length = msgs[1]
                        methods["send"](info["address"], Plugin.__passgenalphanum(int(length)))
                    if msgs[0] == ".passgenspecialchar":
                        length = msgs[1]
                        if int(length) >= 4:
                            methods["send"](
                                info["address"],
                                Plugin.__passgenspecialchar(int(length)),
                            )
                        else:
                            raise Exception("Length of password should be greater than 4.")
        except Exception as e:
            print("woops plug", e)



"""
[age.py]
Age Plugin

[Author]
Justin Walker

[About]
Given birthday it returns your age.

[Commands]
>>> .age <<day>> <<month>> <<year>>
returns age in yr/mo/day format.
"""

from datetime import date


class Plugin:
    def __init__(self):
        pass

    def __age(day, mo, yr):
        years = date.today().year - yr
        months = date.today().month - mo
        days = date.today().day - day
        if months < 0 or (months == 0 and day < 0):
            years -= 1

        msg = f"You are {years}yrs. {abs(months)}mo. and {abs(days)} days old."
        return msg

    def run(self, incoming, methods, info, bot_info):
        try:
            msgs = info["args"][1:][0].split()
            print(len(msgs))
            if info["command"] == "PRIVMSG" and msgs[0] == ".age":
                if len(msgs) == 4 and len(msgs[1]) < 3 and len(msgs[2]) < 3 and len(msgs[3]) == 4:
                    day = int(msgs[1])
                    mo = int(msgs[2])
                    yr = int(msgs[3])
                    methods["send"](info["address"], Plugin.__age(day, mo, yr))
                else:
                    methods["send"](
                        info["address"],
                        "Incorrect format. Should be '.age dy mo year -- -- ----'",
                    )
        except Exception as e:
            print("woops plugin error: ", e)
